import os
import tempfile
import unittest

from avocado.core import sysinfo

from .. import temp_dir_prefix


class SysinfoTest(unittest.TestCase):

    def setUp(self):
        prefix = temp_dir_prefix(__name__, self, 'setUp')
        self.tmpdir = tempfile.TemporaryDirectory(prefix=prefix)

    def test_loggables_equal(self):
        cmd1 = sysinfo.Command("ls -l")
        cmd2 = sysinfo.Command("ls -l")
        self.assertEqual(cmd1, cmd2)
        file1 = sysinfo.Logfile("/proc/cpuinfo")
        file2 = sysinfo.Logfile("/proc/cpuinfo")
        self.assertEqual(file1, file2)

    def test_loggables_not_equal(self):
        cmd1 = sysinfo.Command("ls -l")
        cmd2 = sysinfo.Command("ls -la")
        self.assertNotEqual(cmd1, cmd2)
        file1 = sysinfo.Logfile("/proc/cpuinfo")
        file2 = sysinfo.Logfile("/etc/fstab")
        self.assertNotEqual(file1, file2)

    def test_loggables_set(self):
        container = set()
        cmd1 = sysinfo.Command("ls -l")
        cmd2 = sysinfo.Command("ls -l")
        cmd3 = sysinfo.Command("ps -ef")
        cmd4 = sysinfo.Command("uname -a")
        file1 = sysinfo.Command("/proc/cpuinfo")
        file2 = sysinfo.Command("/etc/fstab")
        file3 = sysinfo.Command("/etc/fstab")
        file4 = sysinfo.Command("/etc/fstab")
        # From the above 8 objects, only 5 are unique loggables.
        container.add(cmd1)
        container.add(cmd2)
        container.add(cmd3)
        container.add(cmd4)
        container.add(file1)
        container.add(file2)
        container.add(file3)
        container.add(file4)

        self.assertEqual(len(container), 5)

    def test_logger_job(self):
        jobdir = os.path.join(self.tmpdir.name, 'job')
        sysinfo_logger = sysinfo.SysInfo(basedir=jobdir)
        sysinfo_logger.start()
        self.assertTrue(os.path.isdir(jobdir))
        self.assertGreaterEqual(len(os.listdir(jobdir)), 1,
                                "Job does not have 'pre' dir")
        job_predir = os.path.join(jobdir, 'pre')
        self.assertTrue(os.path.isdir(job_predir))
        sysinfo_logger.end()
        job_postdir = os.path.join(jobdir, 'post')
        self.assertTrue(os.path.isdir(job_postdir))

    def test_logger_test(self):
        testdir = os.path.join(self.tmpdir.name, 'job', 'test1')
        sysinfo_logger = sysinfo.SysInfo(basedir=testdir)
        sysinfo_logger.start()
        self.assertTrue(os.path.isdir(testdir))
        self.assertGreaterEqual(len(os.listdir(testdir)), 1,
                                "Test does not have 'pre' dir")
        test_predir = os.path.join(testdir, 'pre')
        self.assertTrue(os.path.isdir(test_predir))
        sysinfo_logger.end()
        self.assertGreaterEqual(len(os.listdir(testdir)), 2,
                                "Test does not have 'pre' dir")
        test_postdir = os.path.join(testdir, 'post')
        self.assertTrue(os.path.isdir(test_postdir))

    def tearDown(self):
        self.tmpdir.cleanup()


if __name__ == '__main__':
    unittest.main()



"""
Please don't get inspired by this ugly code
"""
import sys

sys.stdout.write("Direct output to stdout\n")
sys.stderr.write("Direct output to stderr\n")
input("I really want some input on each import")
sys.stdin = 'This is my __COOL__ stdin'
sys.stdout = 'my stdout'
sys.stderr = 'my stderr'
sys.exit(-1)    # Exit even on import



# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See LICENSE for more details.

"""
Avocado Utilities exit codes.

These codes are returned on the command-line and may be used by the Avocado
command-line utilities.
"""

#: The utility finished successfully
UTILITY_OK = 0x0000

#: The utility ran, but needs to signalize a fail.
UTILITY_FAIL = 0x0001

#: Utility generic crash
UTILITY_GENERIC_CRASH = -1



# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See LICENSE for more details.
#
# Copyright: 2016 Red Hat, Inc.
# Author: Lukas Doktor <ldoktor@redhat.com>
"""
avocado.utils.stacktrace unittests
"""

import pickle
import re
import unittest

from avocado.utils import stacktrace


class Unpickable:
    """
    Dummy class which does not support pickling
    """

    def __getstate__(self):
        raise pickle.PickleError


class InClassUnpickable:
    """
    Dummy class containing unpickable object inside itself
    """

    def __init__(self):
        self.troublemaker = Unpickable()


class ListWithUnpickableAttribute(list):
    """
    Dummy list class containing also unpickable attribute
    """

    def __init__(self, *args, **kwargs):
        self.__troublemaker = Unpickable()
        super(ListWithUnpickableAttribute, self).__init__(*args, **kwargs)


class TestUnpickableObject(unittest.TestCase):

    """
    Basic selftests for `avocado.utils.stacktrace.str_unpickable_object
    """

    def test_raises(self):
        self.assertRaises(ValueError, stacktrace.str_unpickable_object,
                          ([{"foo": set([])}]))

    def test_basic(self):
        """ Basic usage """
        def check(exps, obj):
            """ Search exps in the output of str_unpickable_object(obj) """
            act = stacktrace.str_unpickable_object(obj)
            for exp in exps:
                if not re.search(exp, act):
                    self.fail("%r no match in:\n%s" % (exp, act))
        check(["this => .*Unpickable"], Unpickable())
        check([r"this\[0\]\[0\]\[foo\]\.troublemaker => .*Unpickable"],
              [[{"foo": InClassUnpickable()}]])
        check([r"this\[foo\] => \[1, 2, 3\]"],
              {"foo": ListWithUnpickableAttribute([1, 2, 3])})
        check([r"this\[foo\]\[3\] => .*Unpickable"],
              {"foo": ListWithUnpickableAttribute([1, 2, 3, Unpickable()])})
        check([r"this\[2\] => .*Unpickable",
               r"this\[3\]\[foo\].troublemaker => .*Unpickable",
               r"this\[4\]\[0\].troublemaker => .*Unpickable"],
              [1, 2, Unpickable(), {"foo": InClassUnpickable(), "bar": None},
               ListWithUnpickableAttribute(ListWithUnpickableAttribute(
                   [InClassUnpickable()]))])


if __name__ == '__main__':
    unittest.main()



import unittest.mock

from avocado.utils import disk, process

LSBLK_OUTPUT = b'''
{
   "blockdevices": [
      {"name": "/dev/vda", "maj:min": "252:0", "rm": "0", "size": "6G", "ro": "0", "type": "disk", "mountpoint": null,
         "children": [
            {"name": "/dev/vda1", "maj:min": "252:1", "rm": "0", "size": "1M", "ro": "0", "type": "part", "mountpoint": null},
            {"name": "/dev/vda2", "maj:min": "252:2", "rm": "0", "size": "1G", "ro": "0", "type": "part", "mountpoint": "/boot"},
            {"name": "/dev/vda3", "maj:min": "252:3", "rm": "0", "size": "615M", "ro": "0", "type": "part", "mountpoint": "[SWAP]"},
            {"name": "/dev/vda4", "maj:min": "252:4", "rm": "0", "size": "4.4G", "ro": "0", "type": "part", "mountpoint": "/"}
         ]
      }
   ]
}'''


PROC_FILESYSTEMS = (
    'nodev   dax\n' +
    'nodev   bpf\n' +
    'nodev   pipefs\n' +
    'nodev   hugetlbfs\n' +
    'nodev   devpts\n' +
    '        ext3'
)

PROC_MOUNTS = (
    "/dev/mapper/fedora-root / ext4 rw,seclabel,relatime 0 0\n" +
    "/dev/mapper/fedora-home /home ext2 rw,seclabel,relatime 0 0\n" +
    "sysfs /sys sysfs rw,seclabel,nosuid,nodev,noexec,relatime 0 0\n" +
    "proc /proc proc rw,nosuid,nodev,noexec,relatime 0 0"
)


class Disk(unittest.TestCase):

    def test_empty(self):
        mock_result = process.CmdResult(
            command='lsblk --json --paths --inverse',
            stdout=b'{"blockdevices": []}')
        with unittest.mock.patch('avocado.utils.disk.process.run',
                                 return_value=mock_result):
            self.assertEqual(disk.get_disks(), [])

    def test_disks(self):
        mock_result = process.CmdResult(
            command='lsblk --json --paths --inverse',
            stdout=LSBLK_OUTPUT)
        with unittest.mock.patch('avocado.utils.disk.process.run',
                                 return_value=mock_result):
            self.assertEqual(disk.get_disks(), ['/dev/vda'])

    def test_get_filesystems(self):
        expected_fs = ['dax', 'bpf', 'pipefs', 'hugetlbfs', 'devpts', 'ext3']
        open_mocked = unittest.mock.mock_open(read_data=PROC_FILESYSTEMS)
        with unittest.mock.patch('builtins.open', open_mocked):
            self.assertEqual(sorted(expected_fs),
                             sorted(disk.get_available_filesystems()))

    def test_get_filesystem_type_default_root(self):
        open_mocked = unittest.mock.mock_open(read_data=PROC_MOUNTS)
        with unittest.mock.patch('builtins.open', open_mocked):
            self.assertEqual('ext4', disk.get_filesystem_type())

    def test_get_filesystem_type(self):
        open_mocked = unittest.mock.mock_open(read_data=PROC_MOUNTS)
        with unittest.mock.patch('builtins.open', open_mocked):
            self.assertEqual('ext2', disk.get_filesystem_type(mount_point='/home'))


if __name__ == '__main__':
    unittest.main()



# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See LICENSE for more details.
#
# Copyright: Red Hat Inc. 2013-2014
# Author: Lucas Meneghel Rodrigues <lmr@redhat.com>

__all__ = ['MAJOR', 'MINOR', 'VERSION']

import pkg_resources

try:
    VERSION = pkg_resources.get_distribution("avocado-framework").version
except pkg_resources.DistributionNotFound:
    VERSION = "unknown.unknown"

MAJOR, MINOR = VERSION.split('.')



# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See LICENSE for more details.
#
# Copyright: Red Hat, Inc. 2016
# Author: Lukas Doktor <ldoktor@redhat.com>
"""
TAP output module.
"""

import os

from avocado.core.output import LOG_UI
from avocado.core.parser import FileOrStdoutAction
from avocado.core.plugin_interfaces import CLI, Init, ResultEvents
from avocado.core.settings import settings


def file_log_factory(log_file):
    """
    Generates a function which simulates writes to logger and outputs to file

    :param log_file: The output file
    """
    def writeln(msg, *writeargs):
        """
        Format msg and append '\n'
        """
        if writeargs:
            try:
                msg %= writeargs
            except TypeError as details:
                raise TypeError("%s: msg='%s' args='%s'" %
                                (details, msg, writeargs))
        ret = log_file.write(msg + "\n")
        return ret
    return writeln


class TAPResult(ResultEvents):

    """
    TAP output class
    """

    name = 'tap'
    description = "TAP - Test Anything Protocol results"

    def __init__(self, config):  # pylint: disable=W0613
        self.__logs = []
        self.__open_files = []
        self.config = config
        output = self.config.get('job.run.result.tap.output')
        if output == '-':
            log = LOG_UI.debug
            self.__logs.append(log)
        elif output is not None:
            log = open(output, "w", 1)
            self.__open_files.append(log)
            self.__logs.append(file_log_factory(log))
        self.__include_logs = self.config.get('job.run.result.tap.include_logs')
        self.is_header_printed = False

    def __write(self, msg, *writeargs):
        """
        Pass through the message to all registered log functions
        """
        for log in self.__logs:
            log(msg, *writeargs)

    def __flush(self):
        """
        Force-flush the output to opened files.
        """
        for opened_file in self.__open_files:
            opened_file.flush()
            os.fsync(opened_file)

    def pre_tests(self, job):
        """
        Log the test plan
        """
        # Should we add default results.tap?
        if self.config.get('job.run.result.tap.enabled'):
            log = open(os.path.join(job.logdir, 'results.tap'), "w", 1)
            self.__open_files.append(log)
            self.__logs.append(file_log_factory(log))

    def start_test(self, result, state):
        pass

    def end_test(self, result, state):
        """
        Log the test status and details
        """
        if not self.is_header_printed:
            self.__write("1..%d", result.tests_total)
            self.__flush()
            self.is_header_printed = True

        status = state.get("status", "ERROR")
        name = state.get("name")
        if not name:
            name = "Unknown"
        else:
            name = name.name + name.str_variant
            name.replace('#', '_')  # Name must not contain #
            if name[0].isdigit():   # Name must not start with digit
                name = "_" + name

        # First log the system output
        if self.__include_logs:
            self.__write("# debug.log of %s:", name)
            with open(state.get("logfile"), "r") as logfile_obj:
                for line in logfile_obj:
                    self.__write("#   %s", line.rstrip())

        # Then the status
        if status == "PASS":
            self.__write("ok %s %s", result.tests_run, name)
        elif status == "WARN":
            self.__write("ok %s %s  # Warnings were printed into warn log",
                         result.tests_run, name)
        elif status in ("SKIP", "CANCEL"):
            self.__write("ok %s %s  # SKIP %s", result.tests_run, name,
                         state.get("fail_reason"))
        else:
            self.__write("not ok %s %s", result.tests_run, name)
        self.__flush()

    def test_progress(self, progress=False):
        pass

    def post_tests(self, job):
        if job.interrupted_reason is not None:
            for pending_test in range(job.result.tests_run + 1, job.result.tests_total + 1):
                self.__write("ok %s # SKIP %s", pending_test, job.interrupted_reason)
            self.__flush()

        for open_file in self.__open_files:
            open_file.close()


class TAPInit(Init):

    name = 'TAP'
    description = "TAP - Test Anything Protocol - result plugin initilization"

    def initialize(self):
        section = 'job.run.result.tap'
        help_msg = ('Enable TAP result output and write it to FILE. Use '
                    '"-" to redirect to standard output.')
        settings.register_option(
            section=section,
            key='output',
            help_msg=help_msg,
            default=None)

        help_msg = ('Enables default TAP result in the job results directory. '
                    'File will be named "results.tap"')
        settings.register_option(
            section=section,
            key='enabled',
            key_type=bool,
            default=True,
            help_msg=help_msg)

        help_msg = 'Include test logs as comments in TAP output.'
        settings.register_option(
            section=section,
            key='include_logs',
            default=False,
            key_type=bool,
            help_msg=help_msg)


class TAP(CLI):

    """
    TAP Test Anything Protocol output avocado plugin
    """

    name = 'TAP'
    description = "TAP - Test Anything Protocol results"

    def configure(self, parser):
        cmd_parser = parser.subcommands.choices.get('run', None)
        if cmd_parser is None:
            return

        settings.add_argparser_to_option(
            namespace='job.run.result.tap.output',
            metavar='FILE',
            action=FileOrStdoutAction,
            parser=cmd_parser,
            long_arg='--tap')

        settings.add_argparser_to_option(
            namespace='job.run.result.tap.enabled',
            parser=cmd_parser,
            long_arg='--disable-tap-job-result')

        settings.add_argparser_to_option(
            namespace='job.run.result.tap.include_logs',
            parser=cmd_parser,
            long_arg='--tap-include-logs')

    def run(self, config):
        pass



import stat
import unittest

from avocado.core import loader, tags
from avocado.core.nrunner import Runnable
from avocado.utils import script

#: What is commonly known as "0664" or "u=rw,g=rw,o=r"
DEFAULT_NON_EXEC_MODE = (stat.S_IRUSR | stat.S_IWUSR |
                         stat.S_IRGRP | stat.S_IWGRP |
                         stat.S_IROTH)


AVOCADO_TEST_TAGS = """#!/usr/bin/env python
from avocado import Test
from avocado import main

import time

class DisabledTest(Test):
    '''
    :avocado: disable
    :avocado: tags=fast,net
    '''
    def test_disabled(self):
        pass

class FastTest(Test):
    '''
    :avocado: tags=fast
    '''
    def test_fast(self):
        '''
        :avocado: tags=net
        '''
        pass

    def test_fast_other(self):
        '''
        :avocado: tags=net
        '''
        pass

class SlowTest(Test):
    '''
    :avocado: tags=slow,disk
    '''
    def test_slow(self):
        time.sleep(1)

class SlowUnsafeTest(Test):
    '''
    :avocado: tags=slow,disk,unsafe
    '''
    def test_slow_unsafe(self):
        time.sleep(1)

class SafeTest(Test):
    '''
    :avocado: tags=safe
    '''
    def test_safe(self):
        pass

class SafeX86Test(Test):
    '''
    :avocado: tags=safe,arch:x86_64
    '''
    def test_safe_x86(self):
        pass

class NoTagsTest(Test):
    def test_no_tags(self):
        pass

if __name__ == "__main__":
    main()
"""


AVOCADO_TEST_OK = """#!/usr/bin/env python
from avocado import Test
from avocado import main

class PassTest(Test):
    def test(self):
        pass

if __name__ == "__main__":
    main()
"""


class TagFilter(unittest.TestCase):

    def setUp(self):
        with script.TemporaryScript('tags.py',
                                    AVOCADO_TEST_TAGS,
                                    'avocado_loader_unittest',
                                    DEFAULT_NON_EXEC_MODE) as test_script:
            this_loader = loader.FileLoader(None, {})
            self.test_suite = this_loader.discover(test_script.path,
                                                   loader.DiscoverMode.ALL)

    def test_no_tag_filter(self):
        self.assertEqual(len(self.test_suite), 7)
        self.assertEqual(self.test_suite[0][0], 'FastTest')
        self.assertEqual(self.test_suite[0][1]['methodName'], 'test_fast')
        self.assertEqual(self.test_suite[1][0], 'FastTest')
        self.assertEqual(self.test_suite[1][1]['methodName'], 'test_fast_other')
        self.assertEqual(self.test_suite[2][0], 'SlowTest')
        self.assertEqual(self.test_suite[2][1]['methodName'], 'test_slow')
        self.assertEqual(self.test_suite[3][0], 'SlowUnsafeTest')
        self.assertEqual(self.test_suite[3][1]['methodName'], 'test_slow_unsafe')
        self.assertEqual(self.test_suite[4][0], 'SafeTest')
        self.assertEqual(self.test_suite[4][1]['methodName'], 'test_safe')
        self.assertEqual(self.test_suite[5][0], 'SafeX86Test')
        self.assertEqual(self.test_suite[5][1]['methodName'], 'test_safe_x86')
        self.assertEqual(self.test_suite[6][0], 'NoTagsTest')
        self.assertEqual(self.test_suite[6][1]['methodName'], 'test_no_tags')

    def test_filter_fast_net(self):
        filtered = tags.filter_test_tags(self.test_suite, ['fast,net'], False, False)
        self.assertEqual(len(filtered), 2)
        self.assertEqual(filtered[0][0], 'FastTest')
        self.assertEqual(filtered[0][1]['methodName'], 'test_fast')
        self.assertEqual(filtered[1][0], 'FastTest')
        self.assertEqual(filtered[1][1]['methodName'], 'test_fast_other')

    def test_filter_fast_net_include_empty(self):
        filtered = tags.filter_test_tags(self.test_suite, ['fast,net'], True, False)
        self.assertEqual(len(filtered), 3)
        self.assertEqual(filtered[0][0], 'FastTest')
        self.assertEqual(filtered[0][1]['methodName'], 'test_fast')
        self.assertEqual(filtered[1][0], 'FastTest')
        self.assertEqual(filtered[1][1]['methodName'], 'test_fast_other')
        self.assertEqual(filtered[2][0], 'NoTagsTest')
        self.assertEqual(filtered[2][1]['methodName'], 'test_no_tags')

    def test_filter_arch(self):
        filtered = tags.filter_test_tags(self.test_suite, ['arch'], False, False)
        self.assertEqual(len(filtered), 1)
        self.assertEqual(filtered[0][0], 'SafeX86Test')
        self.assertEqual(filtered[0][1]['methodName'], 'test_safe_x86')

    def test_filter_arch_include_empty(self):
        filtered = tags.filter_test_tags(self.test_suite, ['arch'], True, False)
        self.assertEqual(len(filtered), 2)
        self.assertEqual(filtered[0][0], 'SafeX86Test')
        self.assertEqual(filtered[0][1]['methodName'], 'test_safe_x86')
        self.assertEqual(filtered[1][0], 'NoTagsTest')
        self.assertEqual(filtered[1][1]['methodName'], 'test_no_tags')

    def test_filter_arch_x86_64(self):
        filtered = tags.filter_test_tags(self.test_suite, ['arch:x86_64'], False, False)
        self.assertEqual(len(filtered), 1)
        self.assertEqual(filtered[0][0], 'SafeX86Test')
        self.assertEqual(filtered[0][1]['methodName'], 'test_safe_x86')

    def test_filter_arch_other(self):
        filtered = tags.filter_test_tags(self.test_suite, ['arch:ppc64'], False, False)
        self.assertEqual(len(filtered), 0)

    def test_filter_arch_other_include_empty_key(self):
        filtered = tags.filter_test_tags(self.test_suite, ['arch:ppc64'], False, True)
        self.assertEqual(len(filtered), 5)
        self.assertEqual(filtered[0][0], 'FastTest')
        self.assertEqual(filtered[0][1]['methodName'], 'test_fast')
        self.assertEqual(filtered[1][0], 'FastTest')
        self.assertEqual(filtered[1][1]['methodName'], 'test_fast_other')
        self.assertEqual(filtered[2][0], 'SlowTest')
        self.assertEqual(filtered[2][1]['methodName'], 'test_slow')
        self.assertEqual(filtered[3][0], 'SlowUnsafeTest')
        self.assertEqual(filtered[3][1]['methodName'], 'test_slow_unsafe')
        self.assertEqual(filtered[4][0], 'SafeTest')
        self.assertEqual(filtered[4][1]['methodName'], 'test_safe')

    def test_filter_arch_other_include_empty_flat_and_key(self):
        filtered = tags.filter_test_tags(self.test_suite, ['arch:ppc64'], True, True)
        self.assertEqual(len(filtered), 6)
        self.assertEqual(filtered[0][0], 'FastTest')
        self.assertEqual(filtered[0][1]['methodName'], 'test_fast')
        self.assertEqual(filtered[1][0], 'FastTest')
        self.assertEqual(filtered[1][1]['methodName'], 'test_fast_other')
        self.assertEqual(filtered[2][0], 'SlowTest')
        self.assertEqual(filtered[2][1]['methodName'], 'test_slow')
        self.assertEqual(filtered[3][0], 'SlowUnsafeTest')
        self.assertEqual(filtered[3][1]['methodName'], 'test_slow_unsafe')
        self.assertEqual(filtered[4][0], 'SafeTest')
        self.assertEqual(filtered[4][1]['methodName'], 'test_safe')
        self.assertEqual(filtered[5][0], 'NoTagsTest')
        self.assertEqual(filtered[5][1]['methodName'], 'test_no_tags')

    def test_filter_fast_net__slow_disk_unsafe(self):
        filtered = tags.filter_test_tags(self.test_suite,
                                         ['fast,net',
                                          'slow,disk,unsafe'],
                                         False, False)
        self.assertEqual(len(filtered), 3)
        self.assertEqual(filtered[0][0], 'FastTest')
        self.assertEqual(filtered[0][1]['methodName'], 'test_fast')
        self.assertEqual(filtered[1][0], 'FastTest')
        self.assertEqual(filtered[1][1]['methodName'], 'test_fast_other')
        self.assertEqual(filtered[2][0], 'SlowUnsafeTest')
        self.assertEqual(filtered[2][1]['methodName'], 'test_slow_unsafe')

    def test_filter_fast_net__slow_disk(self):
        filtered = tags.filter_test_tags(self.test_suite,
                                         ['fast,net',
                                          'slow,disk'],
                                         False, False)
        self.assertEqual(len(filtered), 4)
        self.assertEqual(filtered[0][0], 'FastTest')
        self.assertEqual(filtered[0][1]['methodName'], 'test_fast')
        self.assertEqual(filtered[1][0], 'FastTest')
        self.assertEqual(filtered[1][1]['methodName'], 'test_fast_other')
        self.assertEqual(filtered[2][0], 'SlowTest')
        self.assertEqual(filtered[2][1]['methodName'], 'test_slow')
        self.assertEqual(filtered[3][0], 'SlowUnsafeTest')
        self.assertEqual(filtered[3][1]['methodName'], 'test_slow_unsafe')

    def test_filter_not_fast_not_slow(self):
        filtered = tags.filter_test_tags(self.test_suite,
                                         ['-fast,-slow'],
                                         False)
        self.assertEqual(len(filtered), 2)
        self.assertEqual(filtered[0][0], 'SafeTest')
        self.assertEqual(filtered[0][1]['methodName'], 'test_safe')
        self.assertEqual(filtered[1][0], 'SafeX86Test')
        self.assertEqual(filtered[1][1]['methodName'], 'test_safe_x86')

    def test_filter_not_fast_not_slow_include_empty(self):
        filtered = tags.filter_test_tags(self.test_suite,
                                         ['-fast,-slow'],
                                         True)
        self.assertEqual(len(filtered), 3)
        self.assertEqual(filtered[0][0], 'SafeTest')
        self.assertEqual(filtered[0][1]['methodName'], 'test_safe')
        self.assertEqual(filtered[1][0], 'SafeX86Test')
        self.assertEqual(filtered[1][1]['methodName'], 'test_safe_x86')
        self.assertEqual(filtered[2][0], 'NoTagsTest')
        self.assertEqual(filtered[2][1]['methodName'], 'test_no_tags')

    def test_filter_not_fast_not_slow_not_safe(self):
        filtered = tags.filter_test_tags(self.test_suite,
                                         ['-fast,-slow,-safe'],
                                         False)
        self.assertEqual(len(filtered), 0)

    def test_filter_not_fast_not_slow_not_safe_others_dont_exist(self):
        filtered = tags.filter_test_tags(self.test_suite,
                                         ['-fast,-slow,-safe',
                                          'does,not,exist'],
                                         False)
        self.assertEqual(len(filtered), 0)

    def test_load_tags(self):
        tags_map = {
            'FastTest.test_fast': {'fast': None, 'net': None},
            'FastTest.test_fast_other': {'fast': None, 'net': None},
            'SlowTest.test_slow': {'slow': None, 'disk': None},
            'SlowUnsafeTest.test_slow_unsafe': {'slow': None,
                                                'disk': None,
                                                'unsafe': None},
            'SafeTest.test_safe': {'safe': None},
            'SafeX86Test.test_safe_x86': {'safe': None,
                                          'arch': set(['x86_64'])},
            'NoTagsTest.test_no_tags': {}
        }

        for _, info in self.test_suite:
            name = info['name'].split(':', 1)[1]
            self.assertEqual(info['tags'], tags_map[name])
            del(tags_map[name])
        self.assertEqual(len(tags_map), 0)


class TagFilter2(unittest.TestCase):

    def test_filter_tags_include_empty(self):
        with script.TemporaryScript('passtest.py',
                                    AVOCADO_TEST_OK,
                                    'avocado_loader_unittest',
                                    DEFAULT_NON_EXEC_MODE) as test_script:
            this_loader = loader.FileLoader(None, {})
            test_suite = this_loader.discover(test_script.path,
                                              loader.DiscoverMode.ALL)
        self.assertEqual([], tags.filter_test_tags(test_suite, [], False, False))
        self.assertEqual(test_suite,
                         tags.filter_test_tags(test_suite, [], True, False))


class ParseFilterByTags(unittest.TestCase):

    def test_must(self):
        self.assertEqual(tags._parse_filter_by_tags(['foo,bar,baz']),
                         [(set(['foo', 'bar', 'baz']), set([]))])

    def test_must_must_not(self):
        self.assertEqual(tags._parse_filter_by_tags(['foo,-bar,baz']),
                         [(set(['foo', 'baz']), set(['bar']))])

    def test_musts_must_nots(self):
        self.assertEqual(tags._parse_filter_by_tags(['foo,bar,baz',
                                                     '-FOO,-BAR,-BAZ']),
                         [(set(['foo', 'bar', 'baz']), set([])),
                          (set([]), set(['FOO', 'BAR', 'BAZ']))])


class FilterRunnable(unittest.TestCase):

    def test_no_tags(self):
        runnable = Runnable('noop', None)
        self.assertFalse(tags.filter_test_tags_runnable(runnable, []))



from avocado import Test


class NastyException:

    """ Please never use something like this!!! (old-style exception) """

    def __init__(self, msg):
        self.msg = msg

    def __str__(self):
        return self.msg


class FailTest(Test):

    """
    This test raises old-style-class exception

    :avocado: tags=failure_expected
    """

    def test(self):
        """
        Avocado should report this as TestError.
        """
        raise NastyException("Nasty-string-like-exception")   # pylint: disable=E0710



# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See LICENSE for more details.
#
# Copyright: Red Hat Inc. 2019
# Author: Cleber Rosa <crosa@redhat.com>

from .settings import settings


def add_tag_filter_args(parser):
    section = 'filter.by_tags'
    group = parser.add_argument_group('filtering parameters')
    settings.register_option(section=section,
                             key='tags',
                             help_msg='Filter tests based on tags',
                             action='append',
                             key_type=list,
                             default=[],
                             metavar='TAGS',
                             parser=group,
                             short_arg='-t',
                             long_arg='--filter-by-tags',
                             allow_multiple=True)

    help_msg = ('Include all tests without tags during filtering. This '
                'effectively means they will be kept in the test suite '
                'found previously to filtering.')
    settings.register_option(section=section,
                             key='include_empty',
                             default=False,
                             key_type=bool,
                             help_msg=help_msg,
                             parser=group,
                             long_arg='--filter-by-tags-include-empty',
                             allow_multiple=True)

    help_msg = ('Include all tests that do not have a matching key in its '
                'key:val tags. This effectively means those tests will be '
                'kept in the test suite found previously to filtering.')
    settings.register_option(section=section,
                             key='include_empty_key',
                             default=False,
                             key_type=bool,
                             help_msg=help_msg,
                             parser=group,
                             long_arg='--filter-by-tags-include-empty-key',
                             allow_multiple=True)



from avocado import Test
from avocado.utils import kernel


class LinuxBuildTest(Test):

    """
    Execute the Linux Build test.

    :avocado: tags=requires_c_compiler

    :param linux_version: kernel version to be built
    :param linux_config: name of the config file located in a data directory
    """

    def setUp(self):
        kernel_src_url = self.params.get('linux_src_url', default='https://www.kernel.org/pub/linux/kernel/v5.x/')
        kernel_version = self.params.get('linux_version', default='5.4.1')
        linux_config = self.params.get('linux_config', default=None)
        self.do_kernel_install = self.params.get('do_kernel_install', default=None)
        if linux_config is not None:
            linux_config = self.get_data(linux_config)
        if linux_config is None:
            self.cancel('Test is missing data file %s' % linux_config)

        self.linux_build = kernel.KernelBuild(kernel_version,
                                              linux_config,
                                              self.workdir,
                                              self.cache_dirs)
        self.linux_build.download(kernel_src_url)
        self.linux_build.uncompress()
        self.linux_build.configure()

    def test(self):
        self.linux_build.build(True if self.do_kernel_install is not None else False)
        if self.do_kernel_install is not None:
            self.linux_build.install()



# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See LICENSE for more details.
#
# Copyright: Red Hat Inc. 2013-2014
# Author: Lucas Meneghel Rodrigues <lmr@redhat.com>

import multiprocessing
import os

from . import process


def configure(path, configure=None):  # pylint: disable=W0621
    """
    Configures the source tree for a subsequent build

    Most source directories coming from official released tarballs
    will have a "configure" script, but source code snapshots may have
    "autogen.sh" instead (which usually creates and runs a "configure"
    script itself).  This function will attempt to run the first one
    found (if a configure script name not given explicitly).

    :param configure: the name of the configure script (None for trying to
                      find one automatically)
    :type configure: str or None
    :returns: the configure script exit status, or None if no script was
              found and executed
    """
    cwd = os.getcwd()
    try:
        os.chdir(path)
        if configure is not None:
            return process.run(os.path.join(path, configure)).exit_status

        candidates = ['autogen.sh', 'configure']
        for configure in candidates:
            if os.access(configure, os.R_OK | os.X_OK):
                return process.run(os.path.join(path, configure)).exit_status
    finally:
        os.chdir(cwd)


def run_make(path, make='make', extra_args='', process_kwargs=None):  # pylint: disable=W0621
    """
    Run make, adding MAKEOPTS to the list of options.

    :param path: directory from where to run make
    :param make: what make command name to use.
    :param extra_args: extra command line arguments to pass to make.
    :param process_kwargs: Additional key word arguments to the underlying
                           process running the make.
    :returns: the make command result object
    """
    cwd = os.getcwd()
    os.chdir(path)
    cmd = make

    if process_kwargs is None:
        process_kwargs = {}
    # Set default number of jobs as ncpus + 1
    os_makeflags = os.environ.get('MAKEFLAGS', '')
    if process_kwargs.get('env') is None:
        process_kwargs['env'] = {}
    args_makeflags = process_kwargs['env'].get('MAKEFLAGS', '')

    if '-j' not in os_makeflags and '-j' not in args_makeflags:
        args_makeflags += ' -j%s' % (multiprocessing.cpu_count() + 1)
        process_kwargs['env'].update({'MAKEFLAGS': args_makeflags})

    makeopts = os.environ.get('MAKEOPTS', '')
    if makeopts:
        cmd += ' %s' % makeopts
    if extra_args:
        cmd += ' %s' % extra_args

    make_process = process.run(cmd, **process_kwargs)
    os.chdir(cwd)
    return make_process


def make(path, make='make', env=None, extra_args='', ignore_status=None,  # pylint: disable=W0621
         allow_output_check=None, process_kwargs=None):
    """
    Run make, adding MAKEOPTS to the list of options.

    :param make: what make command name to use.
    :param env: dictionary with environment variables to be set before
                calling make (e.g.: CFLAGS).
    :param extra: extra command line arguments to pass to make.
    :param allow_output_check: Whether to log the command stream outputs
                               (stdout and stderr) of the make process in
                               the test stream files. Valid values: 'stdout',
                               for allowing only standard output, 'stderr',
                               to allow only standard error, 'all',
                               to allow both standard output and error,
                               and 'none', to allow none to be
                               recorded (default). The default here is
                               'none', because usually we don't want
                               to use the compilation output as a reference
                               in tests.
    :type allow_output_check: str
    :returns: exit status of the make process
    """

    kwargs = dict(env=env,
                  ignore_status=ignore_status,
                  allow_output_check=allow_output_check)
    if process_kwargs is not None:
        kwargs.update(process_kwargs)
    result = run_make(path, make, extra_args, kwargs)

    return result.exit_status



"""
Functional tests for fetch_asset core test method
"""


import os
import tempfile
import unittest
import warnings

from avocado.core import exit_codes
from avocado.utils import process

from .. import AVOCADO, get_temporary_config

TEST_TEMPLATE = r"""
from avocado import Test

class FetchAsset(Test):
    def test_fetch_asset(self):
{content}
"""


class FetchAsset(unittest.TestCase):
    """
    Functional test class for fetch_asset core method
    """

    def setUp(self):
        """
        Setup configuration file and folders
        """
        warnings.simplefilter("ignore", ResourceWarning)
        self.base_dir, self.mapping, self.config_file = get_temporary_config(
            __name__, self, 'setUp')

        self.asset_dir = os.path.join(self.mapping['cache_dir'],
                                      'by_location',
                                      'foo')
        os.makedirs(self.asset_dir)

    def test_asset_fetch_find_success(self):
        """
        Test ends successfully
        Asset is found in the cache
        """
        assetname = 'foo.tgz'
        localpath = os.path.join(self.asset_dir, assetname)
        with open(localpath, 'w') as f:
            f.write('Test!')
        url = 'file://%s' % localpath
        fetch_content = r"""
        foo = self.fetch_asset(
            '%s',
            locations='%s',
            find_only=True)
        print(foo)
        """ % (assetname, url)
        test_content = TEST_TEMPLATE.format(content=fetch_content)
        test_file = tempfile.NamedTemporaryFile(suffix=".py", delete=False)
        test_file.write(test_content.encode())
        test_file.close()

        expected_rc = exit_codes.AVOCADO_ALL_OK
        cmd_line = "%s --config %s run %s " % (AVOCADO,
                                               self.config_file.name,
                                               test_file.name)
        result = process.run(cmd_line)
        os.remove(localpath)
        self.assertEqual(expected_rc, result.exit_status)

    def test_asset_fetch_find_fail(self):
        """
        Test fails
        Asset is not found in the cache
        """
        fake_assetname = 'fake_foo.tgz'
        localpath = os.path.join(self.asset_dir, fake_assetname)
        fake_url = 'file://%s' % localpath
        fetch_content = r"""
        foo = self.fetch_asset(
            '%s',
            locations='%s',
            find_only=True)
        if foo is None:
            raise OSError('Asset not found')
        """ % (fake_assetname, fake_url)
        test_content = TEST_TEMPLATE.format(content=fetch_content)
        test_file = tempfile.NamedTemporaryFile(suffix=".py", delete=False)
        test_file.write(test_content.encode())
        test_file.close()

        expected_rc = exit_codes.AVOCADO_TESTS_FAIL
        expected_stdout = "not found in the cache"

        cmd_line = "%s --config %s run %s " % (AVOCADO,
                                               self.config_file.name,
                                               test_file.name)

        result = process.run(cmd_line, ignore_status=True)
        self.assertEqual(expected_rc, result.exit_status)
        self.assertIn(expected_stdout, result.stdout_text)

    def test_asset_fetch_find_fail_cancel(self):
        """
        Test cancels
        Asset is not found in the cache
        """
        fake_assetname = 'fake_foo.tgz'
        localpath = os.path.join(self.asset_dir, fake_assetname)
        fake_url = 'file://%s' % localpath
        fetch_content = r"""
        foo = self.fetch_asset(
            '%s',
            locations='%s',
            find_only=True,
            cancel_on_missing=True)
        """ % (fake_assetname, fake_url)
        test_content = TEST_TEMPLATE.format(content=fetch_content)
        test_file = tempfile.NamedTemporaryFile(suffix=".py", delete=False)
        test_file.write(test_content.encode())
        test_file.close()

        expected_rc = exit_codes.AVOCADO_ALL_OK
        expected_stdout = "Missing asset"

        cmd_line = "%s --config %s run %s " % (AVOCADO,
                                               self.config_file.name,
                                               test_file.name)

        result = process.run(cmd_line, ignore_status=True)
        self.assertEqual(expected_rc, result.exit_status)
        self.assertIn(expected_stdout, result.stdout_text)

    def test_asset_fetch_cancel(self):
        """
        Test cancels
        Failed to fetch asset
        """
        fake_assetname = 'fake_foo.tgz'
        localpath = os.path.join(self.asset_dir, fake_assetname)
        fake_url = 'file://%s' % localpath
        fetch_content = r"""
        foo = self.fetch_asset(
            '%s',
            locations='%s',
            cancel_on_missing=True)
        """ % (fake_assetname, fake_url)
        test_content = TEST_TEMPLATE.format(content=fetch_content)
        test_file = tempfile.NamedTemporaryFile(suffix=".py", delete=False)
        test_file.write(test_content.encode())
        test_file.close()

        expected_rc = exit_codes.AVOCADO_ALL_OK
        expected_stdout = "Missing asset"

        cmd_line = "%s --config %s run %s " % (AVOCADO,
                                               self.config_file.name,
                                               test_file.name)

        result = process.run(cmd_line, ignore_status=True)
        self.assertEqual(expected_rc, result.exit_status)
        self.assertIn(expected_stdout, result.stdout_text)

    def tearDown(self):
        os.remove(self.config_file.name)
        self.base_dir.cleanup()


if __name__ == '__main__':
    unittest.main()



import avocado as foo
import avocado as bar   # pylint: disable=W0404

from avocado import Test as Foo
from avocado import Test as Bar     # pylint: disable=W0404


class Test1(foo.Test):
    def test1(self):
        pass


class Test2(bar.Test):
    def test2(self):
        pass


class Test3(Foo):
    def test3(self):
        pass


class Test4(Bar):
    def test4(self):
        pass



# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See LICENSE for more details.
#
# Copyright: Red Hat Inc. 2014
# Author: Ruda Moura <rmoura@redhat.com>
"""
Module to help extract and create compressed archives.
"""

import gzip
import logging
import lzma
import os
import platform
import stat
import tarfile
import zipfile

LOG = logging.getLogger(__name__)


#: The first two bytes that all gzip files start with
GZIP_MAGIC = b'\037\213'


def is_gzip_file(path):
    """
    Checks if file given by path has contents that suggests gzip file
    """
    with open(path, 'rb') as gzip_file:
        return gzip_file.read(len(GZIP_MAGIC)) == GZIP_MAGIC


def gzip_uncompress(path, output_path):
    """
    Uncompress a gzipped file at path, to either a file or dir at output_path
    """
    with gzip.GzipFile(filename=path, mode='rb') as input_file:
        if os.path.isdir(output_path):
            basename = os.path.basename(path)
            if basename.endswith('.gz'):
                basename = basename[:-3]
            output_path = os.path.join(output_path, basename)
        with open(output_path, 'wb') as output_file:
            while True:
                chunk = input_file.read(4096)
                if not chunk:
                    break
                output_file.write(chunk)
        return output_path


def is_lzma_file(path):
    """
    Checks if file given by path has contents that suggests lzma file
    """
    with lzma.LZMAFile(path, 'rb') as lzma_file:
        try:
            _ = lzma_file.read(1)
        except lzma.LZMAError:
            return False
        except EOFError:
            return False
    return True


def lzma_uncompress(path, output_path=None, force=False):
    """
    Extracts a XZ compressed file to the same directory.
    """
    if output_path is None:
        output_path = os.path.splitext(path)[0]
    elif os.path.isdir(output_path):
        basename = os.path.basename(path)
        if basename.endswith('.xz'):
            basename = os.path.splitext(basename)[0]
        output_path = os.path.join(output_path, basename)
    if not force and os.path.exists(output_path):
        return output_path
    with lzma.open(path, 'rb') as file_obj:
        with open(output_path, 'wb') as newfile_obj:
            newfile_obj.write(file_obj.read())
    return output_path


class ArchiveException(Exception):
    """
    Base exception for all archive errors.
    """


class ArchiveFile:

    """
    Class that represents an Archive file.

    Archives are ZIP files or Tarballs.
    """

    # extension info: is_zip, is_tar, zipfile|tarfile, +mode
    _extension_table = {
        '.zip': (True, False, zipfile.ZipFile, ''),
        '.tar': (False, True, tarfile.open, ''),
        '.tar.gz': (False, True, tarfile.open, ':gz'),
        '.tgz': (False, True, tarfile.open, ':gz'),
        '.tar.bz2': (False, True, tarfile.open, ':bz2'),
        '.tbz2': (False, True, tarfile.open, ':bz2'),
        '.xz': (False, True, tarfile.open, ':xz')}

    def __init__(self, filename, mode='r'):
        """
        Creates an instance of :class:`ArchiveFile`.

        :param filename: the archive file name.
        :param mode: file mode, `r` read, `w` write.
        """
        self.filename = filename
        self.mode = mode
        engine = None
        for ext in ArchiveFile._extension_table:
            if filename.endswith(ext):
                (self.is_zip,
                 self.is_tar,
                 engine,
                 extra_mode) = ArchiveFile._extension_table[ext]
        if engine is not None:
            self.mode += extra_mode
            self._engine = engine(self.filename, self.mode)
        else:
            raise ArchiveException('file is not an archive')

    def __repr__(self):
        return "ArchiveFile('%s', '%s')" % (self.filename, self.mode)

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_value, exc_traceback):
        if self._engine is not None:
            self.close()

    @classmethod
    def open(cls, filename, mode='r'):
        """
        Creates an instance of :class:`ArchiveFile`.

        :param filename: the archive file name.
        :param mode: file mode, `r` read, `w` write.
        """
        return cls(filename, mode)

    def add(self, filename, arcname=None):
        """
        Add file to the archive.

        :param filename: file to archive.
        :param arcname: alternative name for the file in the archive.
        """
        if self.is_zip:
            self._engine.write(filename, arcname, zipfile.ZIP_DEFLATED)
        else:
            self._engine.add(filename, arcname)

    def list(self):
        """
        List files to the standard output.
        """
        if self.is_zip:
            self._engine.printdir()
        else:
            self._engine.list()

    def extract(self, path='.'):
        """
        Extract all files from the archive.

        :param path: destination path.
        :return: the first member of the archive, a file or directory or None
                 if the archive is empty
        """
        self._engine.extractall(path)
        if self.is_zip:
            self._update_zip_extra_attrs(path)
            files = self._engine.namelist()
            if files:
                return files[0].strip(os.sep)
            else:
                return None

        files = self._engine.getnames()
        if files:
            return files[0]
        return None

    def _update_zip_extra_attrs(self, dst_dir):
        if platform.system() != "Linux":
            LOG.warning("Attr handling in zip files only supported on Linux.")
            return
        # Walk all files and re-create files as symlinks
        for path, info in self._engine.NameToInfo.items():
            dst = os.path.join(dst_dir, path)
            if not os.path.exists(dst):
                LOG.warning("One or more files in the ZIP archive '%s' could "
                            "not be found after extraction. Their paths are "
                            "probably stored in unsupported format and their "
                            "attributes are not going to be updated",
                            self.filename)
                return
            attr = info.external_attr >> 16
            if attr & stat.S_IFLNK == stat.S_IFLNK:
                dst = os.path.join(dst_dir, path)
                if not os.path.islink(dst):
                    # Link created as an ordinary file containing the dst path
                    with open(dst, 'r') as dst_path:
                        src = dst_path.read()
                else:
                    # Link is already there and could be outdated. Let's read
                    # the original destination from the zip file.
                    src = self._engine.read(path)
                os.remove(dst)
                os.symlink(src, dst)
                continue    # Don't override any other attributes on links
            mode = attr & 511   # Mask only permissions
            if mode and mode != 436:  # If mode is stored and is not default
                os.chmod(dst, mode)

    def close(self):
        """
        Close archive.
        """
        self._engine.close()


def is_archive(filename):
    """
    Test if a given file is an archive.

    :param filename: file to test.
    :return: `True` if it is an archive.
    """
    return (zipfile.is_zipfile(filename) or tarfile.is_tarfile(filename) or
            is_gzip_file(filename) or is_lzma_file(filename))


def compress(filename, path):
    """
    Compress files in an archive.

    :param filename: archive file name.
    :param path: origin directory path to files to compress. No
                 individual files allowed.
    """
    with ArchiveFile.open(filename, 'w') as x:
        if os.path.isdir(path):
            for root, _, files in os.walk(path):
                for name in files:
                    newroot = root.replace(path, '')
                    x.add(os.path.join(root, name),
                          os.path.join(newroot, name))
        elif os.path.isfile(path):
            x.add(path, os.path.basename(path))


def uncompress(filename, path):
    """
    Extract files from an archive.

    :param filename: archive file name.
    :param path: destination path to extract to.
    """
    is_tar = tarfile.is_tarfile(filename)
    if is_gzip_file(filename) and not is_tar:
        return gzip_uncompress(filename, path)
    elif is_lzma_file(filename) and not is_tar:
        return lzma_uncompress(filename, path)
    else:
        with ArchiveFile.open(filename) as x:
            return x.extract(path)


# Some aliases
create = compress
extract = uncompress



# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See LICENSE for more details.
#
# Copyright: Red Hat Inc. 2014
# Author: Lucas Meneghel Rodrigues <lmr@redhat.com>
# Author: Lukas Doktor <ldoktor@redhat.com>
"""
This file contains tools for (not only) Avocado developers.
"""
import logging
import os
import time

# Use this for debug logging
LOGGER = logging.getLogger("avocado.app.debug")

# measure_duration global storage
__MEASURE_DURATION = {}


def measure_duration(func):
    """
    Use this as decorator to measure duration of the function execution.
    The output is "Function $name: ($current_duration, $accumulated_duration)"
    """
    def wrapper(*args, **kwargs):
        """ Wrapper function """
        start = time.time()
        try:
            return func(*args, **kwargs)
        finally:
            duration = time.time() - start
            __MEASURE_DURATION[func] = (__MEASURE_DURATION.get(func, 0) +
                                        duration)
            LOGGER.debug("PERF: %s: (%ss, %ss)", func, duration,
                         __MEASURE_DURATION[func])
    return wrapper


def log_calls_class(length=None):
    """
    Use this as decorator to log the function methods' calls.
    :param length: Max message length
    """
    def wrap(orig_cls):
        for key, attr in orig_cls.__dict__.items():
            if callable(attr):
                setattr(orig_cls, key,
                        _log_calls(attr, length, orig_cls.__name__))
        return orig_cls
    return wrap


def _log_calls(func, length=None, cls_name=None):
    """
    log_calls wrapper function
    """
    def wrapper(*args, **kwargs):
        """ Wrapper function """
        msg = ("CALL: %s:%s%s(%s, %s)"
               % (os.path.relpath(func.__code__.co_filename),
                  cls_name, func.__name__,
                  ", ".join([str(_) for _ in args]),
                  ", ".join(["%s=%s" % (key, value)
                             for key, value in kwargs.items()])))
        if length:
            msg = msg[:length]
        LOGGER.debug(msg)
        return func(*args, **kwargs)
    if cls_name:
        cls_name = cls_name + "."
    return wrapper


def log_calls(length=None, cls_name=None):
    """
    Use this as decorator to log the function call altogether with arguments.
    :param length: Max message length
    :param cls_name: Optional class name prefix
    """
    def wrap(func):
        return _log_calls(func, length, cls_name)
    return wrap



import json
import unittest

from avocado.utils import download


class CirrusCI(unittest.TestCase):

    def test(self):
        url = 'https://api.cirrus-ci.com/github/avocado-framework/avocado.json'
        http_response = download.url_open(url)
        self.assertEqual(http_response.code, 200)
        content = http_response.read()
        data = json.loads(content)
        self.assertIn('status', data)
        self.assertEqual(data['status'], 'passing')



import os
import tempfile
import unittest.mock

from avocado.core import settings

from .. import temp_dir_prefix


class Base(unittest.TestCase):

    def _get_temp_dirs_mapping_and_config(self):
        """
        Creates a temporary bogus base data dir

        And returns a dictionary containing the temporary data dir paths and
        a the path to a configuration file contain those same settings
        """
        prefix = temp_dir_prefix(__name__, self, 'setUp')
        base_dir = tempfile.TemporaryDirectory(prefix=prefix)
        test_dir = os.path.join(base_dir.name, 'tests')
        os.mkdir(test_dir)
        mapping = {'base_dir': base_dir.name,
                   'test_dir': test_dir,
                   'data_dir': os.path.join(base_dir.name, 'data'),
                   'logs_dir': os.path.join(base_dir.name, 'logs')}
        temp_settings = ('[datadir.paths]\n'
                         'base_dir = %(base_dir)s\n'
                         'test_dir = %(test_dir)s\n'
                         'data_dir = %(data_dir)s\n'
                         'logs_dir = %(logs_dir)s\n') % mapping
        config_file = tempfile.NamedTemporaryFile('w', delete=False)
        config_file.write(temp_settings)
        config_file.close()
        return (base_dir, mapping, config_file.name)

    def setUp(self):
        (self.base_dir,
         self.mapping,
         self.config_file_path) = self._get_temp_dirs_mapping_and_config()

    def tearDown(self):
        os.unlink(self.config_file_path)
        self.base_dir.cleanup()


class DataDirTest(Base):

    def test_datadir_from_config(self):
        """
        When avocado.conf is present, honor the values coming from it.
        """
        stg = settings.Settings()
        with unittest.mock.patch('avocado.core.stgs', stg):
            import avocado.core
            avocado.core.register_core_options()
        stg.process_config_path(self.config_file_path)
        stg.merge_with_configs()
        with unittest.mock.patch('avocado.core.data_dir.settings', stg):
            from avocado.core import data_dir
            for key in self.mapping.keys():
                data_dir_func = getattr(data_dir, 'get_%s' % key)
                namespace = 'datadir.paths.{}'.format(key)
                self.assertEqual(data_dir_func(), stg.as_dict().get(namespace))

    def test_unique_log_dir(self):
        """
        Tests that multiple queries for a logdir at the same time provides
        unique results.
        """
        from avocado.core import data_dir
        with unittest.mock.patch('avocado.core.data_dir.time.strftime',
                                 return_value="date_would_go_here"):
            logdir = os.path.join(self.mapping['base_dir'], "foor", "bar", "baz")
            path_prefix = os.path.join(logdir, "job-date_would_go_here-")
            uid = "1234567890"*4
            for i in range(7, 40):
                path = data_dir.create_job_logs_dir(logdir, uid)
                self.assertEqual(path, path_prefix + uid[:i])
                self.assertTrue(os.path.exists(path))
            path = data_dir.create_job_logs_dir(logdir, uid)
            self.assertEqual(path, path_prefix + uid + ".0")
            self.assertTrue(os.path.exists(path))
            path = data_dir.create_job_logs_dir(logdir, uid)
            self.assertEqual(path, path_prefix + uid + ".1")
            self.assertTrue(os.path.exists(path))

    def test_get_job_results_dir(self):
        from avocado.core import data_dir
        from avocado.core import job_id

        # First let's mock a jobs results directory
        #

        logs_dir = self.mapping.get('logs_dir')
        self.assertNotEqual(None, logs_dir)
        unique_id = job_id.create_unique_job_id()
        # Expected job results dir
        expected_jrd = data_dir.create_job_logs_dir(logs_dir, unique_id)

        # Now let's test some cases
        #

        self.assertEqual(None,
                         data_dir.get_job_results_dir(expected_jrd, logs_dir),
                         ("If passing a directory reference, it expects the id"
                          "file"))

        # Create the id file.
        id_file_path = os.path.join(expected_jrd, 'id')
        with open(id_file_path, 'w') as id_file:
            id_file.write("%s\n" % unique_id)
            id_file.flush()
            os.fsync(id_file)

        self.assertEqual(expected_jrd,
                         data_dir.get_job_results_dir(expected_jrd, logs_dir),
                         "It should get from the path to the directory")

        results_dirname = os.path.basename(expected_jrd)
        self.assertEqual(None,
                         data_dir.get_job_results_dir(results_dirname,
                                                      logs_dir),
                         "It should not get from a valid path to the directory")

        pwd = os.getcwd()
        os.chdir(logs_dir)
        self.assertEqual(expected_jrd,
                         data_dir.get_job_results_dir(results_dirname,
                                                      logs_dir),
                         "It should get from relative path to the directory")
        os.chdir(pwd)

        self.assertEqual(expected_jrd,
                         data_dir.get_job_results_dir(id_file_path, logs_dir),
                         "It should get from the path to the id file")

        self.assertEqual(expected_jrd,
                         data_dir.get_job_results_dir(unique_id, logs_dir),
                         "It should get from the id")

        another_id = job_id.create_unique_job_id()
        self.assertNotEqual(unique_id, another_id)
        self.assertEqual(None,
                         data_dir.get_job_results_dir(another_id, logs_dir),
                         "It should not get from unexisting job")

        self.assertEqual(expected_jrd,
                         data_dir.get_job_results_dir(unique_id[:7], logs_dir),
                         "It should get from partial id equals to 7 digits")

        self.assertEqual(expected_jrd,
                         data_dir.get_job_results_dir(unique_id[:4], logs_dir),
                         "It should get from partial id less than 7 digits")

        almost_id = unique_id[:7] + ('a' * (len(unique_id) - 7))
        self.assertNotEqual(unique_id, almost_id)
        self.assertEqual(None,
                         data_dir.get_job_results_dir(almost_id, logs_dir),
                         ("It should not get if the id is equal on only"
                          "the first 7 characters"))

        os.symlink(expected_jrd, os.path.join(logs_dir, 'latest'))
        self.assertEqual(expected_jrd,
                         data_dir.get_job_results_dir('latest', logs_dir),
                         "It should get from the 'latest' id")

        stg = settings.Settings()
        with unittest.mock.patch('avocado.core.stgs', stg):
            import avocado.core
            avocado.core.register_core_options()
        stg.process_config_path(self.config_file_path)
        stg.merge_with_configs()
        with unittest.mock.patch('avocado.core.data_dir.settings',
                                 stg):
            self.assertEqual(expected_jrd,
                             data_dir.get_job_results_dir(unique_id),
                             "It should use the default base logs directory")


class AltDataDirTest(Base):

    def test_settings_dir_alternate_dynamic(self):
        """
        Tests that changes to the data_dir settings are applied dynamically

        To guarantee that, first the data_dir module is loaded. Then a new,
        alternate set of data directories are created and set in the
        "canonical" settings location, that is, avocado.core.settings.settings.

        No data_dir module reload should be necessary to get the new locations
        from data_dir APIs.
        """
        # Initial settings with initial data_dir locations
        stg = settings.Settings()
        with unittest.mock.patch('avocado.core.stgs', stg):
            import avocado.core
            avocado.core.register_core_options()
        stg.process_config_path(self.config_file_path)
        stg.merge_with_configs()
        with unittest.mock.patch('avocado.core.data_dir.settings', stg):
            from avocado.core import data_dir
            for key in self.mapping.keys():
                data_dir_func = getattr(data_dir, 'get_%s' % key)
                self.assertEqual(data_dir_func(), self.mapping[key])

        (self.alt_base_dir,  # pylint: disable=W0201
         alt_mapping,
         # pylint: disable=W0201
         self.alt_config_file_path) = self._get_temp_dirs_mapping_and_config()

        # Alternate settings with different data_dir location
        alt_stg = settings.Settings()
        with unittest.mock.patch('avocado.core.stgs', alt_stg):
            import avocado.core
            avocado.core.register_core_options()
        alt_stg.process_config_path(self.alt_config_file_path)
        alt_stg.merge_with_configs()
        with unittest.mock.patch('avocado.core.data_dir.settings', alt_stg):
            for key in alt_mapping.keys():
                data_dir_func = getattr(data_dir, 'get_%s' % key)
                self.assertEqual(data_dir_func(), alt_mapping[key])

    def tearDown(self):
        super(AltDataDirTest, self).tearDown()
        os.unlink(self.alt_config_file_path)
        self.alt_base_dir.cleanup()


if __name__ == '__main__':
    unittest.main()



# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See LICENSE for more details.
#
# Copyright: Red Hat Inc. 2014
# Author: Ruda Moura <rmoura@redhat.com>

import os
import sys

from avocado.core import exit_codes
from avocado.core.output import LOG_UI
from avocado.core.plugin_interfaces import CLI
from avocado.core.settings import settings
from avocado.utils import process


class Wrapper(CLI):

    """
    Implements the '--wrapper' flag for the 'run' subcommand
    """

    name = 'wrapper'
    description = "Implements the '--wrapper' flag for the 'run' subcommand"

    def configure(self, parser):
        run_subcommand_parser = parser.subcommands.choices.get('run', None)
        if run_subcommand_parser is None:
            return

        wrap_group = run_subcommand_parser.add_argument_group(
            'wrapper support')
        help_msg = ('Use a script to wrap executables run by a test. The '
                    'wrapper is either a path to a script (AKA a global '
                    'wrapper) or a path to a script followed by colon symbol '
                    '(:), plus a shell like glob to the target EXECUTABLE. '
                    'Multiple wrapper options are allowed, but only one '
                    'global wrapper can be defined.')
        settings.register_option(section='run.wrapper',
                                 key='wrappers',
                                 default=[],
                                 key_type=list,
                                 help_msg=help_msg,
                                 action='append',
                                 metavar='SCRIPT[:EXECUTABLE]',
                                 parser=wrap_group,
                                 long_arg='--wrapper')

    def run(self, config):
        wraps = config.get('run.wrapper.wrappers')
        if wraps:
            for wrap in wraps:
                if ':' not in wrap:
                    if process.WRAP_PROCESS is None:
                        script = os.path.abspath(wrap)
                        process.WRAP_PROCESS = os.path.abspath(script)
                    else:
                        LOG_UI.error("You can't have multiple global "
                                     "wrappers at once.")
                        sys.exit(exit_codes.AVOCADO_FAIL)
                else:
                    script, cmd = wrap.split(':', 1)
                    script = os.path.abspath(script)
                    process.WRAP_PROCESS_NAMES_EXPR.append((script, cmd))
                if not os.path.exists(script):
                    LOG_UI.error("Wrapper '%s' not found!", script)
                    sys.exit(exit_codes.AVOCADO_FAIL)



# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See LICENSE for more details.
#
# Copyright: Red Hat Inc. 2013-2014
# Author: Lucas Meneghel Rodrigues <lmr@redhat.com>

import hashlib
import io
import logging
import os


def hash_file(filename, size=None, algorithm="md5"):
    """
    Calculate the hash value of filename.

    If size is not None, limit to first size bytes.
    Throw exception if something is wrong with filename.
    Can be also implemented with bash one-liner (assuming ``size%1024==0``)::

        dd if=filename bs=1024 count=size/1024 | sha1sum -

    :param filename: Path of the file that will have its hash calculated.
    :param algorithm: Method used to calculate the hash (default is md5).
    :param size: If provided, hash only the first size bytes of the file.
    :return: Hash of the file, if something goes wrong, return None.
    """
    chunksize = io.DEFAULT_BUFFER_SIZE
    fsize = os.path.getsize(filename)

    if not size or size > fsize:
        size = fsize

    try:
        hash_obj = hashlib.new(algorithm)
    except ValueError as detail:
        logging.error('Returning "None" due to inability to create hash '
                      'object: "%s"', detail)
        return None

    with open(filename, 'rb') as file_to_hash:
        while size > 0:
            if chunksize > size:
                chunksize = size
            data = file_to_hash.read(chunksize)
            if len(data) == 0:
                logging.debug("Nothing left to read but size=%d", size)
                break
            hash_obj.update(data)
            size -= len(data)

    return hash_obj.hexdigest()



import logging
import os

from .. import distro
from .manager import SoftwareManager

log = logging.getLogger('avocado.utils.software_manager')


def install_distro_packages(distro_pkg_map, interactive=False):
    """
    Installs packages for the currently running distribution

    This utility function checks if the currently running distro is a
    key in the distro_pkg_map dictionary, and if there is a list of packages
    set as its value.

    If these conditions match, the packages will be installed using the
    software manager interface, thus the native packaging system if the
    currently running distro.

    :type distro_pkg_map: dict
    :param distro_pkg_map: mapping of distro name, as returned by
        utils.get_os_vendor(), to a list of package names
    :return: True if any packages were actually installed, False otherwise
    """
    if not interactive:
        os.environ['DEBIAN_FRONTEND'] = 'noninteractive'

    result = False
    pkgs = []
    detected_distro = distro.detect()

    distro_specs = [spec for spec in distro_pkg_map if
                    isinstance(spec, distro.Spec)]

    for distro_spec in distro_specs:
        if distro_spec.name != detected_distro.name:
            continue

        if (distro_spec.arch is not None and
                distro_spec.arch != detected_distro.arch):
            continue

        if int(detected_distro.version) < distro_spec.min_version:
            continue

        if (distro_spec.min_release is not None and
                int(detected_distro.release) < distro_spec.min_release):
            continue

        pkgs = distro_pkg_map[distro_spec]
        break

    if not pkgs:
        log.info("No specific distro release package list")

        # when comparing distro names only, fallback to a lowercase version
        # of the distro name is it's more common than the case as detected
        pkgs = distro_pkg_map.get(detected_distro.name, None)
        if not pkgs:
            pkgs = distro_pkg_map.get(detected_distro.name.lower(), None)

        if not pkgs:
            log.error("No generic distro package list")

    if pkgs:
        needed_pkgs = []
        software_manager = SoftwareManager()
        for pkg in pkgs:
            if not software_manager.check_installed(pkg):
                needed_pkgs.append(pkg)
        if needed_pkgs:
            text = ' '.join(needed_pkgs)
            log.info('Installing packages "%s"', text)
            result = software_manager.install(text)
    else:
        log.error("No packages found for %s %s %s %s",
                  detected_distro.name, detected_distro.arch,
                  detected_distro.version, detected_distro.release)
    return result



import io
import logging
import os
import sys
import time
import unittest.mock

from avocado.utils import path, process, script

from .. import (setup_avocado_loggers, skipOnLevelsInferiorThan,
                skipUnlessPathExists)

setup_avocado_loggers()


def probe_binary(binary):
    try:
        return path.find_command(binary)
    except path.CmdNotFoundError:
        return None


ECHO_CMD = probe_binary('echo')
FICTIONAL_CMD = '/usr/bin/fictional_cmd'

REFUSE_TO_DIE = """import signal
import time

for sig in range(64):
    try:
        signal.signal(sig, signal.SIG_IGN)
    except:
        pass

end = time.time() + 120

while time.time() < end:
    time.sleep(1)"""


class TestSubProcess(unittest.TestCase):

    def test_allow_output_check_parameter(self):
        self.assertRaises(ValueError, process.SubProcess,
                          FICTIONAL_CMD, False, "invalid")

    @unittest.mock.patch('avocado.utils.process.SubProcess._init_subprocess')
    @unittest.mock.patch('avocado.utils.process.SubProcess.is_sudo_enabled')
    @unittest.mock.patch('avocado.utils.process.SubProcess.get_pid')
    @unittest.mock.patch('avocado.utils.process.get_children_pids')
    @unittest.mock.patch('avocado.utils.process.run')
    def test_send_signal_sudo_enabled(self, run, get_children, get_pid, sudo, _):
        signal = 1
        pid = 122
        child_pid = 123
        sudo.return_value = True
        get_pid.return_value = pid
        get_children.return_value = [child_pid]

        subprocess = process.SubProcess(FICTIONAL_CMD)
        subprocess.send_signal(signal)

        kill_cmd = 'kill -%d %d'
        calls = [unittest.mock.call(kill_cmd % (signal, child_pid), sudo=True),
                 unittest.mock.call(kill_cmd % (signal, pid), sudo=True)]
        run.assert_has_calls(calls)

    @unittest.mock.patch('avocado.utils.process.SubProcess._init_subprocess')
    @unittest.mock.patch('avocado.utils.process.SubProcess.is_sudo_enabled')
    @unittest.mock.patch('avocado.utils.process.SubProcess.get_pid')
    @unittest.mock.patch('avocado.utils.process.get_children_pids')
    @unittest.mock.patch('avocado.utils.process.run')
    def test_send_signal_sudo_enabled_with_exception(self, run, get_children,
                                                     get_pid, sudo, _):
        signal = 1
        pid = 122
        child_pid = 123
        sudo.return_value = True
        get_pid.return_value = pid
        get_children.return_value = [child_pid]
        run.side_effect = Exception()

        subprocess = process.SubProcess(FICTIONAL_CMD)
        subprocess.send_signal(signal)

        kill_cmd = 'kill -%d %d'
        calls = [unittest.mock.call(kill_cmd % (signal, child_pid), sudo=True),
                 unittest.mock.call(kill_cmd % (signal, pid), sudo=True)]
        run.assert_has_calls(calls)

    @unittest.mock.patch('avocado.utils.process.SubProcess._init_subprocess')
    @unittest.mock.patch('avocado.utils.process.SubProcess.get_pid')
    @unittest.mock.patch('avocado.utils.process.get_owner_id')
    def test_get_user_id(self, get_owner, get_pid, _):
        user_id = 1
        process_id = 123
        get_pid.return_value = process_id
        get_owner.return_value = user_id

        subprocess = process.SubProcess(FICTIONAL_CMD)

        self.assertEqual(subprocess.get_user_id(), user_id)
        get_owner.assert_called_with(process_id)

    @unittest.mock.patch('avocado.utils.process.SubProcess._init_subprocess')
    @unittest.mock.patch('avocado.utils.process.SubProcess.get_pid')
    @unittest.mock.patch('avocado.utils.process.get_owner_id')
    def test_is_sudo_enabled_false(self, get_owner, get_pid, _):
        user_id = 1
        process_id = 123
        get_pid.return_value = process_id
        get_owner.return_value = user_id

        subprocess = process.SubProcess(FICTIONAL_CMD)

        self.assertFalse(subprocess.is_sudo_enabled())
        get_owner.assert_called_with(process_id)

    @unittest.mock.patch('avocado.utils.process.SubProcess._init_subprocess')
    @unittest.mock.patch('avocado.utils.process.SubProcess.get_pid')
    @unittest.mock.patch('avocado.utils.process.get_owner_id')
    def test_is_sudo_enabled_true(self, get_owner, get_pid, _):
        user_id = 0
        process_id = 123
        get_pid.return_value = process_id
        get_owner.return_value = user_id

        subprocess = process.SubProcess(FICTIONAL_CMD)

        self.assertTrue(subprocess.is_sudo_enabled())
        get_owner.assert_called_with(process_id)


def mock_fail_find_cmd(cmd, default=None, check_exec=True):  # pylint: disable=W0613
    path_paths = ["/usr/libexec", "/usr/local/sbin", "/usr/local/bin",
                  "/usr/sbin", "/usr/bin", "/sbin", "/bin"]
    raise path.CmdNotFoundError(cmd, path_paths)


class TestProcessRun(unittest.TestCase):

    @unittest.mock.patch.object(os, 'getuid',
                                unittest.mock.Mock(return_value=1000))
    def test_subprocess_nosudo(self):
        expected_command = 'ls -l'
        p = process.SubProcess(cmd='ls -l')
        self.assertEqual(p.cmd, expected_command)

    @unittest.mock.patch.object(os, 'getuid',
                                unittest.mock.Mock(return_value=0))
    def test_subprocess_nosudo_uid_0(self):
        expected_command = 'ls -l'
        p = process.SubProcess(cmd='ls -l')
        self.assertEqual(p.cmd, expected_command)

    @unittest.mock.patch.object(path, 'find_command',
                                unittest.mock.Mock(return_value='/bin/sudo'))
    @unittest.mock.patch.object(os, 'getuid',
                                unittest.mock.Mock(return_value=1000))
    def test_subprocess_sudo(self):
        expected_command = '/bin/sudo -n ls -l'
        p = process.SubProcess(cmd='ls -l', sudo=True)
        path.find_command.assert_called_once_with('sudo', check_exec=False)
        self.assertEqual(p.cmd, expected_command)

    @unittest.mock.patch.object(path, 'find_command', mock_fail_find_cmd)
    @unittest.mock.patch.object(os, 'getuid',
                                unittest.mock.Mock(return_value=1000))
    def test_subprocess_sudo_no_sudo_installed(self):
        expected_command = 'ls -l'
        p = process.SubProcess(cmd='ls -l', sudo=True)
        self.assertEqual(p.cmd, expected_command)

    @unittest.mock.patch.object(os, 'getuid',
                                unittest.mock.Mock(return_value=0))
    def test_subprocess_sudo_uid_0(self):
        expected_command = 'ls -l'
        p = process.SubProcess(cmd='ls -l', sudo=True)
        self.assertEqual(p.cmd, expected_command)

    @unittest.mock.patch.object(path, 'find_command',
                                unittest.mock.Mock(return_value='/bin/sudo'))
    @unittest.mock.patch.object(os, 'getuid',
                                unittest.mock.Mock(return_value=1000))
    def test_subprocess_sudo_shell(self):
        expected_command = '/bin/sudo -n -s ls -l'
        p = process.SubProcess(cmd='ls -l', sudo=True, shell=True)
        path.find_command.assert_called_once_with('sudo', check_exec=False)
        self.assertEqual(p.cmd, expected_command)

    @unittest.mock.patch.object(path, 'find_command', mock_fail_find_cmd)
    @unittest.mock.patch.object(os, 'getuid',
                                unittest.mock.Mock(return_value=1000))
    def test_subprocess_sudo_shell_no_sudo_installed(self):
        expected_command = 'ls -l'
        p = process.SubProcess(cmd='ls -l', sudo=True, shell=True)
        self.assertEqual(p.cmd, expected_command)

    @unittest.mock.patch.object(os, 'getuid',
                                unittest.mock.Mock(return_value=0))
    def test_subprocess_sudo_shell_uid_0(self):
        expected_command = 'ls -l'
        p = process.SubProcess(cmd='ls -l', sudo=True, shell=True)
        self.assertEqual(p.cmd, expected_command)

    @unittest.mock.patch.object(os, 'getuid',
                                unittest.mock.Mock(return_value=1000))
    def test_run_nosudo(self):
        expected_command = 'ls -l'
        p = process.run(cmd='ls -l', ignore_status=True)
        self.assertEqual(p.command, expected_command)

    @unittest.mock.patch.object(os, 'getuid',
                                unittest.mock.Mock(return_value=0))
    def test_run_nosudo_uid_0(self):
        expected_command = 'ls -l'
        p = process.run(cmd='ls -l', ignore_status=True)
        self.assertEqual(p.command, expected_command)

    @skipUnlessPathExists('/bin/sudo')
    @unittest.mock.patch.object(path, 'find_command',
                                unittest.mock.Mock(return_value='/bin/sudo'))
    @unittest.mock.patch.object(os, 'getuid',
                                unittest.mock.Mock(return_value=1000))
    def test_run_sudo(self):
        expected_command = '/bin/sudo -n ls -l'
        p = process.run(cmd='ls -l', sudo=True, ignore_status=True)
        path.find_command.assert_called_once_with('sudo', check_exec=False)
        self.assertEqual(p.command, expected_command)

    @unittest.mock.patch.object(path, 'find_command', mock_fail_find_cmd)
    @unittest.mock.patch.object(os, 'getuid',
                                unittest.mock.Mock(return_value=1000))
    def test_run_sudo_no_sudo_installed(self):
        expected_command = 'ls -l'
        p = process.run(cmd='ls -l', sudo=True, ignore_status=True)
        self.assertEqual(p.command, expected_command)

    @unittest.mock.patch.object(os, 'getuid',
                                unittest.mock.Mock(return_value=0))
    def test_run_sudo_uid_0(self):
        expected_command = 'ls -l'
        p = process.run(cmd='ls -l', sudo=True, ignore_status=True)
        self.assertEqual(p.command, expected_command)

    @unittest.mock.patch.object(path, 'find_command',
                                unittest.mock.Mock(return_value='/bin/sudo'))
    @unittest.mock.patch.object(os, 'getuid',
                                unittest.mock.Mock(return_value=1000))
    def test_run_sudo_shell(self):
        expected_command = '/bin/sudo -n -s ls -l'
        p = process.run(cmd='ls -l', sudo=True, shell=True, ignore_status=True)
        path.find_command.assert_called_once_with('sudo', check_exec=False)
        self.assertEqual(p.command, expected_command)

    @unittest.mock.patch.object(path, 'find_command', mock_fail_find_cmd)
    @unittest.mock.patch.object(os, 'getuid',
                                unittest.mock.Mock(return_value=1000))
    def test_run_sudo_shell_no_sudo_installed(self):
        expected_command = 'ls -l'
        p = process.run(cmd='ls -l', sudo=True, shell=True, ignore_status=True)
        self.assertEqual(p.command, expected_command)

    @unittest.mock.patch.object(os, 'getuid',
                                unittest.mock.Mock(return_value=0))
    def test_run_sudo_shell_uid_0(self):
        expected_command = 'ls -l'
        p = process.run(cmd='ls -l', sudo=True, shell=True, ignore_status=True)
        self.assertEqual(p.command, expected_command)

    @unittest.skipUnless(ECHO_CMD, "Echo command not available in system")
    def test_run_unicode_output(self):
        # Using encoded string as shlex does not support decoding
        # but the behavior is exactly the same as if shell binary
        # produced unicode
        text = u"Avok\xe1do"
        # Even though code point used is "LATIN SMALL LETTER A WITH ACUTE"
        # (http://unicode.scarfboy.com/?s=u%2B00e1) when encoded to proper
        # utf-8, it becomes two bytes because it is >= 0x80
        # See https://en.wikipedia.org/wiki/UTF-8
        encoded_text = b'Avok\xc3\xa1do'
        self.assertEqual(text.encode('utf-8'), encoded_text)
        self.assertEqual(encoded_text.decode('utf-8'), text)
        cmd = u"%s -n %s" % (ECHO_CMD, text)
        result = process.run(cmd, encoding='utf-8')
        self.assertEqual(result.stdout, encoded_text)
        self.assertEqual(result.stdout_text, text)

    @skipOnLevelsInferiorThan(2)
    def test_run_with_timeout_ugly_cmd(self):
        """
        :avocado: tags=parallel:1
        """
        with script.TemporaryScript("refuse_to_die", REFUSE_TO_DIE) as exe:
            cmd = "%s '%s'" % (sys.executable, exe.path)
            # Wait 1s to set the traps
            res = process.run(cmd, timeout=1, ignore_status=True)
            self.assertLess(res.duration, 100, "Took longer than expected, "
                            "process probably not interrupted by Avocado.\n%s"
                            % res)
            self.assertNotEqual(res.exit_status, 0, "Command finished without "
                                "reporting failure but should be killed.\n%s"
                                % res)

    @skipOnLevelsInferiorThan(2)
    def test_run_with_negative_timeout_ugly_cmd(self):
        """
        :avocado: tags=parallel:1
        """
        with script.TemporaryScript("refuse_to_die", REFUSE_TO_DIE) as exe:
            cmd = "%s '%s'" % (sys.executable, exe.path)
            # Wait 1s to set the traps
            proc = process.SubProcess(cmd)
            proc.start()
            time.sleep(1)
            proc.wait(-1)
            res = proc.result
            self.assertLess(res.duration, 100, "Took longer than expected, "
                            "process probably not interrupted by Avocado.\n%s"
                            % res)
            self.assertNotEqual(res.exit_status, 0, "Command finished without "
                                "reporting failure but should be killed.\n%s"
                                % res)


class MiscProcessTests(unittest.TestCase):

    def test_binary_from_shell(self):
        self.assertEqual("binary", process.binary_from_shell_cmd("binary"))
        res = process.binary_from_shell_cmd("MY_VAR=foo myV4r=123 "
                                            "quote='a b c' QUOTE=\"1 2 && b\" "
                                            "QuOtE=\"1 2\"foo' 3 4' first_bin "
                                            "second_bin VAR=xyz")
        self.assertEqual("first_bin", res)
        res = process.binary_from_shell_cmd("VAR=VALUE 1st_binary var=value "
                                            "second_binary")
        self.assertEqual("1st_binary", res)
        res = process.binary_from_shell_cmd("FOO=bar ./bin var=value")
        self.assertEqual("./bin", res)

    def test_cmd_split(self):
        self.assertEqual(process.cmd_split(''), [])
        self.assertEqual(process.cmd_split("avok\xe1do_test_runner arguments"),
                         ["avok\xe1do_test_runner",
                          "arguments"])

    def test_get_parent_pid(self):
        stat = b'18405 (bash) S 24139 18405 18405 34818 8056 4210688 9792 170102 0 7 11 4 257 84 20 0 1 0 44336493 235409408 4281 18446744073709551615 94723230367744 94723231442728 140723100226000 0 0 0 65536 3670020 1266777851 0 0 0 17 1 0 0 0 0 0 94723233541456 94723233588580 94723248717824 140723100229613 140723100229623 140723100229623 140723100233710 0'
        with unittest.mock.patch('builtins.open',
                                 return_value=io.BytesIO(stat)):
            self.assertTrue(process.get_parent_pid(0), 24139)

    @unittest.skipUnless(sys.platform.startswith('linux'),
                         'Linux specific feature and test')
    def test_get_children_pids(self):
        '''
        Gets the list of children process.  Linux only.
        '''
        self.assertGreaterEqual(len(process.get_children_pids(1)), 1)

    @unittest.mock.patch('avocado.utils.process.os.kill')
    @unittest.mock.patch('avocado.utils.process.get_owner_id')
    def test_safe_kill(self, owner_mocked, kill_mocked):
        owner_id = 1
        process_id = 123
        signal = 1
        owner_mocked.return_value = owner_id

        killed = process.safe_kill(process_id, signal)
        self.assertTrue(killed)
        kill_mocked.assert_called_with(process_id, signal)

    @unittest.mock.patch('avocado.utils.process.os.kill')
    @unittest.mock.patch('avocado.utils.process.get_owner_id')
    def test_safe_kill_with_exception(self, owner_mocked, kill_mocked):
        owner_id = 1
        process_id = 123
        signal = 1
        owner_mocked.return_value = owner_id
        kill_mocked.side_effect = Exception()

        killed = process.safe_kill(process_id, signal)
        self.assertFalse(killed)
        kill_mocked.assert_called_with(process_id, signal)

    @unittest.mock.patch('avocado.utils.process.run')
    @unittest.mock.patch('avocado.utils.process.get_owner_id')
    def test_safe_kill_sudo_enabled(self, owner_mocked, run_mocked):
        owner_id = 0
        process_id = 123
        signal = 1
        owner_mocked.return_value = owner_id
        expected_cmd = 'kill -%d %d' % (signal, process_id)

        killed = process.safe_kill(process_id, signal)
        self.assertTrue(killed)
        run_mocked.assert_called_with(expected_cmd, sudo=True)

    @unittest.mock.patch('avocado.utils.process.run')
    @unittest.mock.patch('avocado.utils.process.get_owner_id')
    def test_safe_kill_sudo_enabled_with_exception(self, owner_mocked, run_mocked):
        owner_id = 0
        process_id = 123
        signal = 1
        owner_mocked.return_value = owner_id
        expected_cmd = 'kill -%d %d' % (signal, process_id)
        run_mocked.side_effect = process.CmdError()

        killed = process.safe_kill(process_id, signal)
        self.assertFalse(killed)
        run_mocked.assert_called_with(expected_cmd, sudo=True)

    @unittest.mock.patch('avocado.utils.process.os.stat')
    def test_process_get_owner_id(self, stat_mock):
        process_id = 123
        owner_user_id = 13
        stat_mock.return_value = unittest.mock.Mock(st_uid=owner_user_id)

        returned_owner_id = process.get_owner_id(process_id)

        self.assertEqual(returned_owner_id, owner_user_id)
        stat_mock.assert_called_with('/proc/%d/' % process_id)

    @unittest.mock.patch('avocado.utils.process.os.stat')
    def test_process_get_owner_id_with_pid_not_found(self, stat_mock):
        process_id = 123
        stat_mock.side_effect = OSError()

        returned_owner_id = process.get_owner_id(process_id)

        self.assertIsNone(returned_owner_id)
        stat_mock.assert_called_with('/proc/%d/' % process_id)

    @unittest.mock.patch('avocado.utils.process.time.sleep')
    @unittest.mock.patch('avocado.utils.process.safe_kill')
    @unittest.mock.patch('avocado.utils.process.get_children_pids')
    def test_kill_process_tree_nowait(self, get_children_pids, safe_kill,
                                      sleep):
        safe_kill.return_value = True
        get_children_pids.return_value = []
        self.assertEqual([1], process.kill_process_tree(1))
        self.assertEqual(sleep.call_count, 0)

    @unittest.mock.patch('avocado.utils.process.safe_kill')
    @unittest.mock.patch('avocado.utils.process.get_children_pids')
    @unittest.mock.patch('avocado.utils.process.time.time')
    @unittest.mock.patch('avocado.utils.process.time.sleep')
    @unittest.mock.patch('avocado.utils.process.pid_exists')
    def test_kill_process_tree_timeout_3s(self, pid_exists, sleep, p_time,
                                          get_children_pids, safe_kill):
        safe_kill.return_value = True
        get_children_pids.return_value = []
        p_time.side_effect = [500, 502, 502, 502, 502, 502, 502,
                              504, 504, 504, 520, 520, 520]
        sleep.return_value = None
        pid_exists.return_value = True
        self.assertRaises(RuntimeError, process.kill_process_tree, 17,
                          timeout=3)
        self.assertLess(p_time.call_count, 10)

    @unittest.mock.patch('avocado.utils.process.safe_kill')
    @unittest.mock.patch('avocado.utils.process.get_children_pids')
    @unittest.mock.patch('avocado.utils.process.time.time')
    @unittest.mock.patch('avocado.utils.process.time.sleep')
    @unittest.mock.patch('avocado.utils.process.pid_exists')
    def test_kill_process_tree_dont_timeout_3s(self, pid_exists, sleep,
                                               p_time, get_children_pids,
                                               safe_kill):
        safe_kill.return_value = True
        get_children_pids.return_value = []
        p_time.side_effect = [500, 502, 502, 502, 502, 502, 502, 502, 502, 503]
        sleep.return_value = None
        pid_exists.side_effect = [True, False]
        self.assertEqual([76], process.kill_process_tree(76, timeout=3))
        self.assertLess(p_time.call_count, 10)

    @unittest.mock.patch('avocado.utils.process.safe_kill')
    @unittest.mock.patch('avocado.utils.process.get_children_pids')
    @unittest.mock.patch('avocado.utils.process.time.sleep')
    @unittest.mock.patch('avocado.utils.process.pid_exists')
    def test_kill_process_tree_dont_timeout_infinity(self, pid_exists, sleep,
                                                     get_children_pids,
                                                     safe_kill):
        safe_kill.return_value = True
        get_children_pids.return_value = []
        sleep.return_value = None
        pid_exists.side_effect = [True, True, True, True, True, False]

        self.assertEqual([31], process.kill_process_tree(31, timeout=-7.354))

        self.assertEqual(pid_exists.call_count, 6)
        self.assertEqual(sleep.call_count, 5)

    @unittest.mock.patch('avocado.utils.process.time.sleep')
    @unittest.mock.patch('avocado.utils.process.safe_kill')
    @unittest.mock.patch('avocado.utils.process.get_children_pids')
    def test_kill_process_tree_children(self, get_children_pids, safe_kill,
                                        sleep):
        safe_kill.return_value = True
        get_children_pids.side_effect = [[53, 12], [78, 58, 41], [], [13],
                                         [], [], []]
        self.assertEqual([31, 53, 78, 58, 13, 41, 12],
                         process.kill_process_tree(31))
        self.assertEqual(sleep.call_count, 0)


class CmdResultTests(unittest.TestCase):

    def test_nasty_str(self):
        result = process.CmdResult("ls", b"unicode_follows: \xc5\xa1",
                                   b"cp1250 follows: \xfd", 1, 2, 3,
                                   "wrong_encoding")
        self.assertEqual(str(result), "command: 'ls'\nexit_status: 1"
                         "\nduration: 2\ninterrupted: False\npid: "
                         "3\nencoding: 'wrong_encoding'\nstdout: "
                         "b'unicode_follows: \\xc5\\xa1'\nstderr: "
                         "b'cp1250 follows: \\xfd'")

    def test_cmd_result_stdout_stderr_bytes(self):
        result = process.CmdResult()
        self.assertTrue(isinstance(result.stdout, bytes))
        self.assertTrue(isinstance(result.stderr, bytes))

    def test_cmd_result_stdout_stderr_text(self):
        result = process.CmdResult()
        self.assertTrue(isinstance(result.stdout_text, str))
        self.assertTrue(isinstance(result.stderr_text, str))

    def test_cmd_result_stdout_stderr_already_text(self):
        result = process.CmdResult()
        result.stdout = "supposed command output, but not as bytes"
        result.stderr = "supposed command error, but not as bytes"
        self.assertEqual(result.stdout, result.stdout_text)
        self.assertEqual(result.stderr, result.stderr_text)

    def test_cmd_result_stdout_stderr_other_type(self):
        result = process.CmdResult()
        result.stdout = None
        result.stderr = None
        self.assertRaises(TypeError, lambda x: result.stdout_text)
        self.assertRaises(TypeError, lambda x: result.stderr_text)


class CmdErrorTests(unittest.TestCase):

    def test_nasty_str(self):
        result = process.CmdResult("ls", b"unicode_follows: \xc5\xa1",
                                   b"cp1250 follows: \xfd", 1, 2, 3,
                                   "wrong_encoding")
        err = process.CmdError("ls", result, "please don't crash")
        self.assertEqual(str(err), "Command 'ls' failed.\nstdout: "
                         "b'unicode_follows: \\xc5\\xa1'\nstderr: "
                         "b'cp1250 follows: \\xfd'\nadditional_info: "
                         "please don't crash")


class FDDrainerTests(unittest.TestCase):

    def test_drain_from_pipe_fd(self):
        read_fd, write_fd = os.pipe()
        result = process.CmdResult()
        fd_drainer = process.FDDrainer(read_fd, result, "test")
        fd_drainer.start()
        for content in (b"foo", b"bar", b"baz", b"foo\nbar\nbaz\n\n"):
            os.write(write_fd, content)
        os.write(write_fd, b"finish")
        os.close(write_fd)
        fd_drainer.flush()
        self.assertEqual(fd_drainer.data.getvalue(),
                         b"foobarbazfoo\nbar\nbaz\n\nfinish")

    def test_log(self):
        class CatchHandler(logging.NullHandler):
            """
            Handler used just to confirm that a logging event happened
            """

            def __init__(self, *args, **kwargs):
                super(CatchHandler, self).__init__(*args, **kwargs)
                self.caught_record = False

            def handle(self, record):
                self.caught_record = True

        read_fd, write_fd = os.pipe()
        result = process.CmdResult()
        logger = logging.getLogger("FDDrainerTests.test_log")
        handler = CatchHandler()
        logger.addHandler(handler)
        logger.setLevel(logging.DEBUG)

        fd_drainer = process.FDDrainer(read_fd, result, "test",
                                       logger=logger, verbose=True)
        fd_drainer.start()
        os.write(write_fd, b"should go to the log\n")
        os.close(write_fd)
        fd_drainer.flush()
        self.assertEqual(fd_drainer.data.getvalue(),
                         b"should go to the log\n")
        self.assertTrue(handler.caught_record)

    def test_flush_on_closed_handler(self):
        handler = logging.StreamHandler(io.StringIO())
        log = logging.getLogger("test_flush_on_closed_handler")
        log.addHandler(handler)
        read_fd, write_fd = os.pipe()
        result = process.CmdResult()
        fd_drainer = process.FDDrainer(read_fd, result, name="test",
                                       stream_logger=log)
        fd_drainer.start()
        os.close(write_fd)
        self.assertIsNotNone(fd_drainer._stream_logger)
        one_stream_closed = False
        for handler in fd_drainer._stream_logger.handlers:
            stream = getattr(handler, 'stream', None)
            if stream is not None:
                if hasattr(stream, 'close'):
                    # force closing the handler's stream to check if
                    # flush will adapt to it
                    stream.close()
                    one_stream_closed = True
        self.assertTrue(one_stream_closed)
        fd_drainer.flush()

    def test_flush_on_handler_with_no_fileno(self):
        handler = logging.StreamHandler(io.StringIO())
        log = logging.getLogger("test_flush_on_handler_with_no_fileno")
        log.addHandler(handler)
        read_fd, write_fd = os.pipe()
        result = process.CmdResult()
        fd_drainer = process.FDDrainer(read_fd, result, name="test",
                                       stream_logger=log)
        fd_drainer.start()
        os.close(write_fd)
        fd_drainer.flush()

    def test_replace_incorrect_characters_in_log(self):
        data = io.StringIO()
        handler = logging.StreamHandler(data)
        log = logging.getLogger("test_replace_incorrect_characters_in_log")
        log.addHandler(handler)
        log.setLevel(logging.DEBUG)
        read_fd, write_fd = os.pipe()
        result = process.CmdResult(encoding='ascii')
        fd_drainer = process.FDDrainer(read_fd, result, name="test",
                                       stream_logger=log, verbose=True)
        fd_drainer.start()
        os.write(write_fd, b"Avok\xc3\xa1do")
        os.close(write_fd)
        fd_drainer._thread.join(60)
        self.assertFalse(fd_drainer._thread.is_alive())
        # \n added by StreamLogger
        self.assertEqual(data.getvalue(), u"Avok\ufffd\ufffddo\n")


class GetCommandOutputPattern(unittest.TestCase):

    @unittest.skipUnless(ECHO_CMD, "Echo command not available in system")
    def test_matches(self):
        res = process.get_command_output_matching("echo foo", "foo")
        self.assertEqual(res, ["foo"])

    @unittest.skipUnless(ECHO_CMD, "Echo command not available in system")
    def test_matches_multiple(self):
        res = process.get_command_output_matching("echo -e 'foo\nfoo\n'", "foo")
        self.assertEqual(res, ["foo", "foo"])

    @unittest.skipUnless(ECHO_CMD, "Echo command not available in system")
    def test_does_not_match(self):
        res = process.get_command_output_matching("echo foo", "bar")
        self.assertEqual(res, [])


if __name__ == "__main__":
    unittest.main()



import os
import shutil
import tempfile

from avocado import Test
from avocado.utils import process, script

CLEAN_TEST = """import os
from avocado import Test

class Clean(Test):
    def test(self):
        path = os.path.expanduser("~/.avocado-has-not-touched-me")
        self.assertFalse(os.path.exists(path))
"""


DIRTY_TEST = """import os
from avocado import Test

class Dirty(Test):
    def test(self):
        self.touch(os.path.expanduser("~/.avocado-has-not-touched-me"))

    @staticmethod
    def touch(name):
        with open(name, 'w'):
            os.utime(name, None)
"""


class VMCleanup(Test):

    """
    Tests the Avocado VM plugin `--vm-cleanup` feature

    The approach chosen here is to have a first run of Avocado making sure that
    a flag file does not exist. A second run of Avocado, now executed with an
    additional `--vm-cleanup` parameter runs a test that creates the flag file.
    Finally, the third and last execution of re-runs the same test as the first
    run, checking that the flag file does *not* exist, that is, the clean up of
    the VM did work.

    Because this test requires a libvirt domain and the hostname/address of
    that virtual machine, these parameters assume no defaults for safety and/or
    security causes. Please edit the vm-cleanup.yaml file with the appropriate
    parameters.
    """

    def setUp(self):
        vm_domain = self.params.get("vm_domain", default=None)
        vm_host = self.params.get("vm_host", default=None)
        if vm_domain is None or vm_host is None:
            self.cancel('Either "vm_domain" or "vm_host" parameters have not '
                        'been given. Please edit the "vm-cleanup.yaml" file '
                        'with the appropriate parameters')

        self.tmpdir = tempfile.mkdtemp(prefix='avocado_' + __name__)
        clean_test = os.path.join(self.tmpdir, 'clean.py')
        self.clean_test_path = script.make_script(clean_test, CLEAN_TEST)
        dirty_test = os.path.join(self.tmpdir, 'dirty.py')
        self.dirty_test_path = script.make_script(dirty_test, DIRTY_TEST)

    def test(self):
        vm_domain = self.params.get("vm_domain", default=None)
        vm_host = self.params.get("vm_host", default=None)
        vm_username = self.params.get("vm_username", default=None)
        vm_password = self.params.get("vm_password", default=None)

        cmd = ('avocado run --sysinfo=off --job-results-dir %s --vm-domain=%s '
               '--vm-host=%s')
        cmd %= (self.tmpdir, vm_domain, vm_host)
        if vm_username:
            cmd += ' --vm-username=%s' % vm_username
        if vm_password:
            cmd += ' --vm-password=%s' % vm_password

        cmd_clean = '%s %s' % (cmd, self.clean_test_path)
        cmd_dirty = '%s --vm-cleanup %s' % (cmd, self.dirty_test_path)

        process.run(cmd_clean)
        process.run(cmd_dirty)
        process.run(cmd_clean)

    def tearDown(self):
        shutil.rmtree(self.tmpdir)



import tempfile
import unittest

from avocado.core.nrunner import Runnable
from avocado.core.parser import HintParser
from avocado.core.resolver import ReferenceResolution
from avocado.core.settings import SettingsError

from .. import skipUnlessPathExists

BAD = """[kinds]
tap = ./tests/*.t
"""

GOOD = """[kinds]
exec-test = /bin/true
[exec-test]
uri = $testpath
"""


class HintTest(unittest.TestCase):
    def setUp(self):
        self.wrong_file = tempfile.NamedTemporaryFile('w', delete=False)
        self.wrong_file.write(BAD)
        self.wrong_file.close()

        self.good_file = tempfile.NamedTemporaryFile('w', delete=False)
        self.good_file.write(GOOD)
        self.good_file.close()

        self.wrong_hint = HintParser(self.wrong_file.name)
        self.good_hint = HintParser(self.good_file.name)

    def test_wrong_parser(self):
        with self.assertRaises(SettingsError) as context:
            self.wrong_hint.validate_kind_section('tap')
        self.assertTrue('Section tap is not defined' in str(context.exception))

    @skipUnlessPathExists('/bin/true')
    def test_types(self):
        res = self.good_hint.get_resolutions()
        self.assertEqual(len(res), 1)
        self.assertIsInstance(res[0], ReferenceResolution)

        resolutions = res[0].resolutions
        self.assertEqual(len(resolutions), 1)
        self.assertIsInstance(resolutions[0], Runnable)

    @skipUnlessPathExists('/bin/true')
    def test_reference_names(self):
        res = self.good_hint.get_resolutions()[0]
        self.assertEqual(res.reference, '/bin/true')
        self.assertEqual(res.resolutions[0].uri, '/bin/true')


if __name__ == '__main__':
    unittest.main()



import os
import time

from avocado import Test


class SleepTenMin(Test):

    """
    Sleeps for 10 minutes

    :avocado: tags=slow

    :param sleep_cycles: How many iterations should be executed
    :param sleep_length: single sleep duration
    :param sleep_method: what method of sleep should be used (builtin|shell)
    """

    def test(self):
        """
        Sleep for length seconds.
        """
        cycles = int(self.params.get('sleep_cycles', default=1))
        length = int(self.params.get('sleep_length', default=600))
        method = self.params.get('sleep_method', default='builtin')

        for _ in range(0, cycles):
            self.log.debug("Sleeping for %.2f seconds", length)
            if method == 'builtin':
                time.sleep(length)
            elif method == 'shell':
                os.system("sleep %s" % length)



# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; specifically version 2 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See LICENSE for more details.
#
# This code was inspired in the autotest project,
# client/shared/utils.py
# Authors: Christian Herdtweck <christian.herdtweck@intra2net.com>

"""

SUMMARY
------------------------------------------------------
Utilities for file tests.


INTERFACE
------------------------------------------------------

"""

import logging
import os
from glob import iglob
from grp import getgrgid, getgrnam
from pwd import getpwnam, getpwuid
from stat import S_IMODE


def check_owner(owner, group, file_name_pattern, check_recursive=False):
    """
    Verifies that given file belongs to given owner and group.

    :param str owner: user that owns of the file
    :param str group: group of the owner of the file
    :param str file_name_pattern: can be a glob
    :param check_recursive: if file_name_pattern matches a directory,
                            recurse into that subdir or not
    :type check_recursive: bool
    :raises: :py:class:`RuntimeError` if file has wrong owner or group
    """
    for file_name in iglob(file_name_pattern):
        actual_id = os.stat(file_name).st_uid
        if actual_id != getpwnam(owner).pw_uid:
            raise RuntimeError(
                'file {0} has wrong owner {1} (should be {2})'
                .format(file_name, getpwuid(actual_id).pw_name, owner))
        actual_id = os.stat(file_name).st_gid
        if actual_id != getgrnam(group).gr_gid:
            raise RuntimeError(
                'file {0} has wrong group {1} (should be {2})'
                .format(file_name, getgrgid(actual_id).gr_name, group))
        logging.debug('checked owner %s:%s of file %s',
                      owner, group, file_name)

        if check_recursive and os.path.isdir(file_name):
            new_pattern = os.path.join(file_name,
                                       os.path.basename(file_name_pattern))
            check_owner(owner, group, new_pattern, True)


def check_permissions(perms, file_name_pattern):
    """
    Verify that a given file has a given numeric permission.

    :param int perms: best given in octal form, e.g. 0o755
    :param str file_name_pattern: can be a glob
    :raises: :py:class:`RuntimeError` if file has wrong permissions
    """
    for file_name in iglob(file_name_pattern):
        actual_perms = S_IMODE(os.stat(file_name).st_mode)
        if perms != actual_perms:
            raise RuntimeError(
                'file {0} has permissions {1} (should be {2})!'
                .format(file_name, oct(actual_perms), oct(perms)))
        logging.debug('checked permissions %s of file %s',
                      oct(perms), file_name)



import json
import os
import unittest

from avocado.core import exit_codes
from avocado.utils import process, script

from .. import AVOCADO, TestCaseTmpDir

STDOUT = b"Hello, \xc4\x9b\xc5\xa1\xc4\x8d\xc5\x99\xc5\xbe\xc3\xbd\xc3\xa1\xc3\xad\xc3\xa9!\n"
STDERR = b"Hello, stderr!\n"

JSON_VARIANTS = """
[{"paths": ["/run/*"],
 "variant": [["/run/params/foo",
            [["/run/params/foo", "p2", "foo2"],
             ["/run/params/foo", "p1", "foo1"]]]],
 "variant_id": "foo"},
{"paths": ["/run/*"],
 "variant": [["/run/params/bar",
            [["/run/params/bar", "p2", "bar2"],
             ["/run/params/bar", "p1", "bar1"]]]],
 "variant_id": "bar"}]
 """

TEST_WITH_SAME_EXPECTED_OUTPUT = """
from avocado import Test
import logging


class PassTest(Test):

    def test_1(self):

        stdout = logging.getLogger('avocado.test.stdout')
        stdout.info('Informational line that will go to stdout')
        stderr = logging.getLogger('avocado.test.stderr')
        stderr.info('Informational line that will go to stderr')

    def test_2(self):

        stdout = logging.getLogger('avocado.test.stdout')
        stdout.info('Informational line that will go to stdout')
        stderr = logging.getLogger('avocado.test.stderr')
        stderr.info('Informational line that will go to stderr')
"""

TEST_WITH_DIFFERENT_EXPECTED_OUTPUT = """
from avocado import Test
import logging


class PassTest(Test):

    def test_1(self):

        stdout = logging.getLogger('avocado.test.stdout')
        stdout.info('Informational line that will go to stdout_1')
        stderr = logging.getLogger('avocado.test.stderr')
        stderr.info('Informational line that will go to stderr_1')

    def test_2(self):

        stdout = logging.getLogger('avocado.test.stdout')
        stdout.info('Informational line that will go to stdout_2')
        stderr = logging.getLogger('avocado.test.stderr')
        stderr.info('Informational line that will go to stderr_2')
"""

TEST_WITH_DIFFERENT_EXPECTED_OUTPUT_VARIANTS = """
from avocado import Test
import logging


class PassTest(Test):

    def test_1(self):
        foo = self.params.get("p1")
        stdout = logging.getLogger('avocado.test.stdout')
        stdout.info('Informational line that will go to stdout_1 %s'%foo)
        stderr = logging.getLogger('avocado.test.stderr')
        stderr.info('Informational line that will go to stderr_1 %s'%foo)
        print("foo %s" %foo)

    def test_2(self):
        bar = self.params.get("p2")
        stdout = logging.getLogger('avocado.test.stdout')
        stdout.info('Informational line that will go to stdout_2 %s'%bar)
        stderr = logging.getLogger('avocado.test.stderr')
        stderr.info('Informational line that will go to stderr_2 %s'%bar)
        print("bar %s" %bar)
"""

TEST_WITH_DIFFERENT_AND_SAME_EXPECTED_OUTPUT = """
from avocado import Test
import logging


class PassTest(Test):

    def test_1(self):

        stdout = logging.getLogger('avocado.test.stdout')
        stdout.info('Informational line that will go to stdout_1')
        stderr = logging.getLogger('avocado.test.stderr')
        stderr.info('Informational line that will go to stderr_1')

    def test_2(self):

        stdout = logging.getLogger('avocado.test.stdout')
        stdout.info('Informational line that will go to stdout_2')
        stderr = logging.getLogger('avocado.test.stderr')
        stderr.info('Informational line that will go to stderr_2')

    def test_3(self):

        stdout = logging.getLogger('avocado.test.stdout')
        stdout.info('Informational line that will go to stdout_2')
        stderr = logging.getLogger('avocado.test.stderr')
        stderr.info('Informational line that will go to stderr_2')
"""


class RunnerSimpleTest(TestCaseTmpDir):

    def assertIsFile(self, path):
        self.assertTrue(os.path.isfile(path))

    def assertIsNotFile(self, path):
        self.assertFalse(os.path.isfile(path))

    def setUp(self):
        super(RunnerSimpleTest, self).setUp()
        content = b"#!/bin/sh\necho -n '%s';echo -n '%s'>&2" % (STDOUT, STDERR)
        self.output_script = script.TemporaryScript(
            'output_check.sh',
            content,
            'avocado_output_check_functional',
            open_mode='wb')
        self.output_script.save()

    def _check_output_record_all(self):
        cmd_line = ('%s run --job-results-dir %s --disable-sysinfo %s '
                    '--output-check-record all'
                    % (AVOCADO, self.tmpdir.name, self.output_script.path))
        result = process.run(cmd_line, ignore_status=True)
        expected_rc = exit_codes.AVOCADO_ALL_OK
        self.assertEqual(result.exit_status, expected_rc,
                         "Avocado did not return rc %d:\n%s" %
                         (expected_rc, result))
        stdout_file = "%s.data/stdout.expected" % self.output_script
        stderr_file = "%s.data/stderr.expected" % self.output_script
        with open(stdout_file, 'rb') as fd_stdout:
            self.assertEqual(fd_stdout.read(), STDOUT)
        with open(stderr_file, 'rb') as fd_stderr:
            self.assertEqual(fd_stderr.read(), STDERR)

    def _check_output_record_combined(self):
        cmd_line = ('%s run --job-results-dir %s --disable-sysinfo %s '
                    '--output-check-record combined'
                    % (AVOCADO, self.tmpdir.name, self.output_script.path))
        result = process.run(cmd_line, ignore_status=True)
        expected_rc = exit_codes.AVOCADO_ALL_OK
        self.assertEqual(result.exit_status, expected_rc,
                         "Avocado did not return rc %d:\n%s" %
                         (expected_rc, result))
        output_file = "%s.data/output.expected" % self.output_script
        with open(output_file, 'rb') as fd_output:
            self.assertEqual(fd_output.read(), STDOUT + STDERR)

    def _setup_simple_test(self, simple_test_content):
        variants_file = os.path.join(self.tmpdir.name, 'variants.json')
        with open(variants_file, 'w') as file_obj:
            file_obj.write(JSON_VARIANTS)
        simple_test = os.path.join(self.tmpdir.name, 'simpletest.py')
        with open(simple_test, 'w') as file_obj:
            file_obj.write(simple_test_content)
        return (simple_test, variants_file)

    def test_output_record_none(self):
        cmd_line = ('%s run --job-results-dir %s --disable-sysinfo %s '
                    '--output-check-record none'
                    % (AVOCADO, self.tmpdir.name, self.output_script.path))
        result = process.run(cmd_line, ignore_status=True)
        expected_rc = exit_codes.AVOCADO_ALL_OK
        self.assertEqual(result.exit_status, expected_rc,
                         "Avocado did not return rc %d:\n%s" %
                         (expected_rc, result))
        self.assertIsNotFile("%s.data/stdout.expected" % self.output_script)
        self.assertIsNotFile("%s.data/stderr.expected" % self.output_script)

    def test_output_record_stdout(self):
        cmd_line = ('%s run --job-results-dir %s --disable-sysinfo %s '
                    '--output-check-record stdout'
                    % (AVOCADO, self.tmpdir.name, self.output_script.path))
        result = process.run(cmd_line, ignore_status=True)
        expected_rc = exit_codes.AVOCADO_ALL_OK
        self.assertEqual(result.exit_status, expected_rc,
                         "Avocado did not return rc %d:\n%s" %
                         (expected_rc, result))
        stdout_file = "%s.data/stdout.expected" % self.output_script
        stderr_file = "%s.data/stderr.expected" % self.output_script
        with open(stdout_file, 'rb') as fd_stdout:
            self.assertEqual(fd_stdout.read(), STDOUT)
        self.assertIsNotFile(stderr_file)

    def test_output_record_and_check(self):
        self._check_output_record_all()
        cmd_line = ('%s run --job-results-dir %s --disable-sysinfo %s'
                    % (AVOCADO, self.tmpdir.name, self.output_script.path))
        result = process.run(cmd_line, ignore_status=True)
        expected_rc = exit_codes.AVOCADO_ALL_OK
        self.assertEqual(result.exit_status, expected_rc,
                         "Avocado did not return rc %d:\n%s" %
                         (expected_rc, result))

    def test_output_record_and_check_combined(self):
        self._check_output_record_combined()
        cmd_line = ('%s run --job-results-dir %s --disable-sysinfo %s'
                    % (AVOCADO, self.tmpdir.name, self.output_script.path))
        result = process.run(cmd_line, ignore_status=True)
        expected_rc = exit_codes.AVOCADO_ALL_OK
        self.assertEqual(result.exit_status, expected_rc,
                         "Avocado did not return rc %d:\n%s" %
                         (expected_rc, result))

    def test_output_tamper_stdout(self):
        self._check_output_record_all()
        tampered_msg = b"I PITY THE FOOL THAT STANDS ON MY WAY!"
        stdout_file = "%s.data/stdout.expected" % self.output_script.path
        with open(stdout_file, 'wb') as stdout_file_obj:
            stdout_file_obj.write(tampered_msg)
        cmd_line = ('%s run --job-results-dir %s --disable-sysinfo %s --xunit -'
                    % (AVOCADO, self.tmpdir.name, self.output_script.path))
        result = process.run(cmd_line, ignore_status=True)
        expected_rc = exit_codes.AVOCADO_TESTS_FAIL
        self.assertEqual(result.exit_status, expected_rc,
                         "Avocado did not return rc %d:\n%s" %
                         (expected_rc, result))
        self.assertIn(tampered_msg, result.stdout)

    def test_output_tamper_combined(self):
        self._check_output_record_combined()
        tampered_msg = b"I PITY THE FOOL THAT STANDS ON MY WAY!"
        output_file = "%s.data/output.expected" % self.output_script.path
        with open(output_file, 'wb') as output_file_obj:
            output_file_obj.write(tampered_msg)
        cmd_line = ('%s run --job-results-dir %s --disable-sysinfo %s --xunit -'
                    % (AVOCADO, self.tmpdir.name, self.output_script.path))
        result = process.run(cmd_line, ignore_status=True)
        expected_rc = exit_codes.AVOCADO_TESTS_FAIL
        self.assertEqual(result.exit_status, expected_rc,
                         "Avocado did not return rc %d:\n%s" %
                         (expected_rc, result))
        self.assertIn(tampered_msg, result.stdout)

    def test_output_diff(self):
        self._check_output_record_all()
        tampered_msg_stdout = b"I PITY THE FOOL THAT STANDS ON STDOUT!"
        tampered_msg_stderr = b"I PITY THE FOOL THAT STANDS ON STDERR!"

        stdout_file = "%s.data/stdout.expected" % self.output_script.path
        with open(stdout_file, 'wb') as stdout_file_obj:
            stdout_file_obj.write(tampered_msg_stdout)

        stderr_file = "%s.data/stderr.expected" % self.output_script.path
        with open(stderr_file, 'wb') as stderr_file_obj:
            stderr_file_obj.write(tampered_msg_stderr)

        cmd_line = ('%s run --job-results-dir %s --disable-sysinfo %s --json -'
                    % (AVOCADO, self.tmpdir.name, self.output_script.path))
        result = process.run(cmd_line, ignore_status=True)
        expected_rc = exit_codes.AVOCADO_TESTS_FAIL
        self.assertEqual(result.exit_status, expected_rc,
                         "Avocado did not return rc %d:\n%s" %
                         (expected_rc, result))

        json_result = json.loads(result.stdout_text)
        job_log = json_result['debuglog']
        stdout_diff = os.path.join(json_result['tests'][0]['logdir'],
                                   'stdout.diff')
        stderr_diff = os.path.join(json_result['tests'][0]['logdir'],
                                   'stderr.diff')

        with open(stdout_diff, 'rb') as stdout_diff_obj:
            stdout_diff_content = stdout_diff_obj.read()
        self.assertIn(b'-I PITY THE FOOL THAT STANDS ON STDOUT!',
                      stdout_diff_content)
        self.assertIn(b'+' + STDOUT, stdout_diff_content)

        with open(stderr_diff, 'rb') as stderr_diff_obj:
            stderr_diff_content = stderr_diff_obj.read()
        self.assertIn(b'-I PITY THE FOOL THAT STANDS ON STDERR!',
                      stderr_diff_content)
        self.assertIn(b'+Hello, stderr!', stderr_diff_content)

        with open(job_log, 'rb') as job_log_obj:
            job_log_content = job_log_obj.read()
        self.assertIn(b'Stdout Diff:', job_log_content)
        self.assertIn(b'-I PITY THE FOOL THAT STANDS ON STDOUT!', job_log_content)
        self.assertIn(b'+' + STDOUT, job_log_content)
        self.assertIn(b'Stdout Diff:', job_log_content)
        self.assertIn(b'-I PITY THE FOOL THAT STANDS ON STDERR!', job_log_content)
        self.assertIn(b'+' + STDERR, job_log_content)

    def test_disable_output_check(self):
        self._check_output_record_all()
        tampered_msg = b"I PITY THE FOOL THAT STANDS ON MY WAY!"
        stdout_file = "%s.data/stdout.expected" % self.output_script.path
        with open(stdout_file, 'wb') as stdout_file_obj:
            stdout_file_obj.write(tampered_msg)
        cmd_line = ('%s run --job-results-dir %s --disable-sysinfo %s '
                    '--disable-output-check --xunit -'
                    % (AVOCADO, self.tmpdir.name, self.output_script.path))
        result = process.run(cmd_line, ignore_status=True)
        expected_rc = exit_codes.AVOCADO_ALL_OK
        self.assertEqual(result.exit_status, expected_rc,
                         "Avocado did not return rc %d:\n%s" %
                         (expected_rc, result))
        self.assertNotIn(tampered_msg, result.stdout)

    def test_merge_records_same_output(self):
        variants_file = os.path.join(self.tmpdir.name, 'variants.json')
        with open(variants_file, 'w') as file_obj:
            file_obj.write(JSON_VARIANTS)
        simple_test = os.path.join(self.tmpdir.name, 'simpletest.py')
        with open(simple_test, 'w') as file_obj:
            file_obj.write(TEST_WITH_SAME_EXPECTED_OUTPUT)
        cmd_line = ('%s run --job-results-dir %s --disable-sysinfo %s '
                    '--output-check-record both --json-variants-load %s' %
                    (AVOCADO, self.tmpdir.name, simple_test, variants_file))
        process.run(cmd_line, ignore_status=True)
        self.assertIsFile("%s.data/stdout.expected" % simple_test)
        self.assertIsFile("%s.data/stderr.expected" % simple_test)

    def test_merge_records_different_output(self):
        simple_test, variants_file = self._setup_simple_test(
            TEST_WITH_DIFFERENT_EXPECTED_OUTPUT)
        cmd_line = ('%s run --job-results-dir %s --disable-sysinfo %s '
                    '--output-check-record both --json-variants-load %s' %
                    (AVOCADO, self.tmpdir.name, simple_test, variants_file))
        process.run(cmd_line, ignore_status=True)
        self.assertIsNotFile("%s.data/stdout.expected" % simple_test)
        self.assertIsNotFile("%s.data/stderr.expected" % simple_test)
        self.assertIsFile("%s.data/PassTest.test_1/stdout.expected" % simple_test)
        self.assertIsFile("%s.data/PassTest.test_1/stderr.expected" % simple_test)
        self.assertIsFile("%s.data/PassTest.test_2/stdout.expected" % simple_test)
        self.assertIsFile("%s.data/PassTest.test_2/stderr.expected" % simple_test)

    def test_merge_records_different_output_variants(self):
        simple_test, variants_file = self._setup_simple_test(
            TEST_WITH_DIFFERENT_EXPECTED_OUTPUT_VARIANTS)
        cmd_line = ('%s run --job-results-dir %s --disable-sysinfo %s '
                    '--output-check-record both --json-variants-load %s' %
                    (AVOCADO, self.tmpdir.name, simple_test, variants_file))
        process.run(cmd_line, ignore_status=True)
        self.assertIsNotFile("%s.data/stdout.expected" % simple_test)
        self.assertIsNotFile("%s.data/stderr.expected" % simple_test)
        self.assertIsNotFile("%s.data/PassTest.test_1/stdout.expected" % simple_test)
        self.assertIsNotFile("%s.data/PassTest.test_1/stderr.expected" % simple_test)
        self.assertIsNotFile("%s.data/PassTest.test_2/stdout.expected" % simple_test)
        self.assertIsNotFile("%s.data/PassTest.test_2/stderr.expected" % simple_test)
        self.assertIsFile("%s.data/PassTest.test_2/bar/stderr.expected" % simple_test)
        self.assertIsFile("%s.data/PassTest.test_2/foo/stderr.expected" % simple_test)

    def test_merge_records_different_and_same_output(self):
        simple_test, variants_file = self._setup_simple_test(
            TEST_WITH_DIFFERENT_AND_SAME_EXPECTED_OUTPUT)
        cmd_line = ('%s run --job-results-dir %s --disable-sysinfo %s '
                    '--output-check-record both --json-variants-load %s' %
                    (AVOCADO, self.tmpdir.name, simple_test, variants_file))
        process.run(cmd_line, ignore_status=True)
        self.assertIsFile("%s.data/stdout.expected" % simple_test)
        self.assertIsFile("%s.data/stderr.expected" % simple_test)
        self.assertIsFile("%s.data/PassTest.test_1/stdout.expected" % simple_test)
        self.assertIsFile("%s.data/PassTest.test_1/stderr.expected" % simple_test)
        self.assertIsNotFile("%s.data/PassTest.test_2/stdout.expected" % simple_test)
        self.assertIsNotFile("%s.data/PassTest.test_2/stderr.expected" % simple_test)

    def tearDown(self):
        super(RunnerSimpleTest, self).tearDown()
        self.output_script.remove()


if __name__ == '__main__':
    unittest.main()



import http.client
import os
import tempfile
import threading
import unittest.mock

from avocado.utils import cloudinit, data_factory, iso9660

from .. import setup_avocado_loggers, temp_dir_prefix

setup_avocado_loggers()


def has_iso_create_write():
    return iso9660.iso9660(os.devnull, ["create", "write"]) is not None


class CloudInit(unittest.TestCase):

    def test_iso_no_create_write(self):
        with unittest.mock.patch('avocado.utils.iso9660.iso9660', return_value=None):
            self.assertRaises(RuntimeError, cloudinit.iso, os.devnull, "INSTANCE_ID")


class CloudInitISO(unittest.TestCase):

    def setUp(self):
        prefix = temp_dir_prefix(__name__, self, 'setUp')
        self.tmpdir = tempfile.TemporaryDirectory(prefix=prefix)

    @unittest.skipUnless(has_iso_create_write(),
                         "system lacks support for creating ISO images")
    def test_iso_no_phone_home(self):
        path = os.path.join(self.tmpdir.name, "cloudinit.iso")
        instance_id = b"INSTANCE_ID"
        username = b"AVOCADO_USER"
        password = b"AVOCADO_PASSWORD"
        cloudinit.iso(path, instance_id, username, password)
        iso = iso9660.iso9660(path)
        self.assertIn(instance_id, iso.read("/meta-data"))
        user_data = iso.read("/user-data")
        self.assertIn(username, user_data)
        self.assertIn(password, user_data)

    def tearDown(self):
        self.tmpdir.cleanup()


class PhoneHome(unittest.TestCase):

    ADDRESS = '127.0.0.1'

    def post_ignore_response(self, url):
        port = self.server.socket.getsockname()[1]
        conn = http.client.HTTPConnection(self.ADDRESS, port)
        conn.request('POST', url)
        try:
            conn.getresponse()
        except Exception:
            pass
        finally:
            conn.close()

    def setUp(self):
        self.instance_id = data_factory.generate_random_string(12)
        self.server = cloudinit.PhoneHomeServer((self.ADDRESS, 0),
                                                self.instance_id)

    def test_phone_home_bad(self):
        self.assertFalse(self.server.instance_phoned_back)
        server_thread = threading.Thread(target=self.server.handle_request)
        server_thread.start()
        self.post_ignore_response('/BAD_INSTANCE_ID')
        self.assertFalse(self.server.instance_phoned_back)

    def test_phone_home_good(self):
        self.assertFalse(self.server.instance_phoned_back)
        server_thread = threading.Thread(target=self.server.handle_request)
        server_thread.start()
        self.post_ignore_response('/' + self.instance_id)
        self.assertTrue(self.server.instance_phoned_back)

    def test_phone_home_bad_good(self):
        self.test_phone_home_bad()
        self.test_phone_home_good()

    def tearDown(self):
        self.server.server_close()


if __name__ == '__main__':
    unittest.main()



# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See LICENSE for more details.
#
# Copyright: Red Hat Inc. 2013-2014
# Authors: Lucas Meneghel Rodrigues <lmr@redhat.com>

"""
Operations with strings (conversion and sanitation).

The unusual name aims to avoid causing name clashes with the stdlib module
string. Even with the dot notation, people may try to do things like

   import string
   ...
   from avocado.utils import string

And not notice until their code starts failing.
"""

import itertools
import locale
import re

#: On import evaluated value representing the system encoding
#: based on system locales using :func:`locale.getpreferredencoding`.
#: Use this value wisely as some files are dumped in different
#: encoding.
ENCODING = locale.getpreferredencoding()

#: String containing all fs-unfriendly chars (Windows-fat/Linux-ext3)
FS_UNSAFE_CHARS = '<>:"/\\|?*;'

# Translate table to replace fs-unfriendly chars
_FS_TRANSLATE = bytes.maketrans(bytes(FS_UNSAFE_CHARS, "ascii"),
                                b'__________')


def bitlist_to_string(data):
    """
    Transform from bit list to ASCII string.

    :param data: Bit list to be transformed
    """
    result = []
    pos = 0
    c = 0
    while pos < len(data):
        c += data[pos] << (7 - (pos % 8))
        if (pos % 8) == 7:
            result.append(c)
            c = 0
        pos += 1
    return ''.join([chr(c) for c in result])


def string_to_bitlist(data):
    """
    Transform from ASCII string to bit list.

    :param data: String to be transformed
    """
    data = [ord(c) for c in data]
    result = []
    for ch in data:
        i = 7
        while i >= 0:
            if ch & (1 << i) != 0:
                result.append(1)
            else:
                result.append(0)
            i -= 1
    return result


def shell_escape(command):
    """
    Escape special characters from a command so that it can be passed
    as a double quoted (" ") string in a (ba)sh command.

    :param command: the command string to escape.

    :return: The escaped command string. The required englobing double
            quotes are NOT added and so should be added at some point by
            the caller.

    See also: http://www.tldp.org/LDP/abs/html/escapingsection.html
    """
    command = command.replace("\\", "\\\\")
    command = command.replace("$", r'\$')
    command = command.replace('"', r'\"')
    command = command.replace('`', r'\`')
    return command


def strip_console_codes(output, custom_codes=None):
    """
    Remove the Linux console escape and control sequences from the console
    output. Make the output readable and can be used for result check. Now
    only remove some basic console codes using during boot up.

    :param output: The output from Linux console
    :type output: string
    :param custom_codes: The codes added to the console codes which is not
                         covered in the default codes
    :type output: string
    :return: the string without any special codes
    :rtype: string
    """
    if "\x1b" not in output:
        return output

    old_word = ""
    return_str = ""
    index = 0
    output = "\x1b[m%s" % output
    console_codes = "%[G@8]|\\[[@A-HJ-MPXa-hl-nqrsu\\`]"
    console_codes += "|\\[[\\d;]+[HJKgqnrm]|#8|\\([B0UK]|\\)"
    if custom_codes is not None and custom_codes not in console_codes:
        console_codes += "|%s" % custom_codes
    while index < len(output):
        tmp_index = 0
        tmp_word = ""
        while (len(re.findall("\x1b", tmp_word)) < 2 and
               index + tmp_index < len(output)):
            tmp_word += output[index + tmp_index]
            tmp_index += 1

        tmp_word = re.sub("\x1b", "", tmp_word)
        index += len(tmp_word) + 1
        if tmp_word == old_word:
            continue
        try:
            special_code = re.findall(console_codes, tmp_word)[0]
        except IndexError:
            if index + tmp_index < len(output):
                raise ValueError("%s is not included in the known console "
                                 "codes list %s" % (tmp_word, console_codes))
            continue
        if special_code == tmp_word:
            continue
        old_word = tmp_word
        return_str += tmp_word[len(special_code):]
    return return_str


def iter_tabular_output(matrix, header=None, strip=False):
    """
    Generator for a pretty, aligned string representation of a nxm matrix.

    This representation can be used to print any tabular data, such as
    database results. It works by scanning the lengths of each element
    in each column, and determining the format string dynamically.

    :param matrix: Matrix representation (list with n rows of m elements).
    :param header: Optional tuple or list with header elements to be displayed.
    :param strip:  Optionally remove trailing whitespace from each row.
    """
    def _get_matrix_with_header():
        return itertools.chain([header], matrix)

    def _get_matrix_no_header():
        return matrix

    if header is None:
        header = []
    if header:
        get_matrix = _get_matrix_with_header
    else:
        get_matrix = _get_matrix_no_header

    lengths = []
    len_matrix = []
    str_matrix = []
    for row in get_matrix():
        len_matrix.append([])
        str_matrix.append([string_safe_encode(column) for column in row])
        for i, column in enumerate(str_matrix[-1]):
            col_len = len(strip_console_codes(column))
            len_matrix[-1].append(col_len)
            try:
                max_len = lengths[i]
                if col_len > max_len:
                    lengths[i] = col_len
            except IndexError:
                lengths.append(col_len)
        # For different no cols we need to calculate `lengths` of the last item
        # but later in `yield` we don't want it in `len_matrix`
        len_matrix[-1] = len_matrix[-1][:-1]

    if strip:
        def str_out(x): return " ".join(x).rstrip()
    else:
        def str_out(x): return " ".join(x)

    for row, row_lens in zip(str_matrix, len_matrix):
        out = []
        padding = [" " * (lengths[i] - row_lens[i])
                   for i in range(len(row_lens))]
        out = ["%s%s" % line for line in zip(row, padding)]
        try:
            out.append(row[-1])
        except IndexError:
            continue    # Skip empty rows
        yield str_out(out)


def tabular_output(matrix, header=None, strip=False):
    """
    Pretty, aligned string representation of a nxm matrix.

    This representation can be used to print any tabular data, such as
    database results. It works by scanning the lengths of each element
    in each column, and determining the format string dynamically.

    :param matrix: Matrix representation (list with n rows of m elements).
    :param header: Optional tuple or list with header elements to be displayed.
    :param strip:  Optionally remove trailing whitespace from each row.
    :return: String with the tabular output, lines separated by unix line feeds.
    :rtype: str
    """
    return "\n".join(iter_tabular_output(matrix, header, strip))


def string_safe_encode(input_str):
    """
    People tend to mix unicode streams with encoded strings. This function
    tries to replace any input with a valid utf-8 encoded ascii stream.

    On Python 3, it's a terrible idea to try to mess with encoding,
    so this function is limited to converting other types into
    strings, such as numeric values that are often the members of a
    matrix.

    :param input_str: possibly unsafe string or other object that can
                      be turned into a string
    :returns: a utf-8 encoded ascii stream
    """
    if not isinstance(input_str, str):
        input_str = str(input_str)
    return input_str


def string_to_safe_path(input_str):
    """
    Convert string to a valid file/dir name.

    This takes a string that may contain characters that are not allowed on
    FAT (Windows) filesystems and/or ext3 (Linux) filesystems, and replaces
    them for safe (boring) underlines.

    It limits the size of the path to be under 255 chars, and make hidden
    paths (starting with ".") non-hidden by making them start with "_".

    :param input_str: String to be converted
    :return: String which is safe to pass as a file/dir name (on recent fs)
    """
    if input_str.startswith("."):
        input_str = "_" + input_str[1:255]
    elif len(input_str) > 255:
        input_str = input_str[:255]

    try:
        return input_str.translate(_FS_TRANSLATE)
    except TypeError:
        # Deal with incorrect encoding
        for bad_chr in FS_UNSAFE_CHARS:
            input_str = input_str.replace(bad_chr, "_")
        return input_str


def is_bytes(data):
    """
    Checks if the data given is a sequence of bytes

    And not a "text" type, that can be of multi-byte characters.
    Also, this does NOT mean a bytearray type.

    :param data: the instance to be checked if it falls under the definition
                 of an array of bytes.
    """
    return isinstance(data, bytes)


def is_text(data):
    """
    Checks if the data given is a suitable for holding text

    That is, if it can hold text that requires more than one byte for
    each character.
    """
    return isinstance(data, str)


def to_text(data, encoding=ENCODING, errors='strict'):
    """
    Convert anything to text decoded text

    When the data is bytes, it's decoded. When it's not of string types
    it's re-formatted into text and returned. Otherwise (it's string)
    it's returned unchanged.

    :param data: data to be transformed into text
    :type data: either bytes or other data that will be returned
                unchanged
    :param encoding: encoding of the data (only used when decoding
                     is necessary)
    :param errors: how to handle encode/decode errors, see:
            https://docs.python.org/3/library/codecs.html#error-handlers
    """
    if is_bytes(data):
        if encoding is None:
            encoding = ENCODING
        return data.decode(encoding, errors=errors)
    elif not isinstance(data, str):
        return str(data)
    return data



import logging
import os

from ... import path as utils_path
from ... import process
from .base import BaseBackend

log = logging.getLogger('avocado.utils.software_manager')


class DpkgBackend(BaseBackend):

    """
    This class implements operations executed with the dpkg package manager.

    dpkg is a lower level package manager, used by higher level managers such
    as apt and aptitude.
    """

    PACKAGE_TYPE = 'deb'
    INSTALLED_OUTPUT = 'install ok installed'

    def __init__(self):
        self.lowlevel_base_cmd = utils_path.find_command('dpkg')

    def check_installed(self, name):
        if os.path.isfile(name):
            n_cmd = self.lowlevel_base_cmd + ' -f ' + name + ' Package'
            name = process.system_output(n_cmd)
        i_cmd = self.lowlevel_base_cmd + " -s " + name
        # Checking if package is installed
        package_status = process.run(i_cmd, ignore_status=True).stdout_text
        dpkg_installed = (self.INSTALLED_OUTPUT in package_status)
        if dpkg_installed:
            return True
        return False

    def list_all(self):
        """
        List all packages available in the system.
        """
        log.debug("Listing all system packages (may take a while)")
        installed_packages = []
        cmd_result = process.run('dpkg -l', verbose=False)
        out = cmd_result.stdout_text.strip()
        raw_list = out.splitlines()[5:]
        for line in raw_list:
            parts = line.split()
            if parts[0] == "ii":  # only grab "installed" packages
                installed_packages.append("%s-%s" % (parts[1], parts[2]))
        return installed_packages

    def list_files(self, package):
        """
        List files installed by package [package].

        :param package: Package name.
        :return: List of paths installed by package.
        """
        if os.path.isfile(package):
            l_cmd = self.lowlevel_base_cmd + ' -c ' + package
        else:
            l_cmd = self.lowlevel_base_cmd + ' -l ' + package
        return process.system_output(l_cmd).split('\n')



from unittest import TestCase

from avocado.core.nrunner import Runnable, Task
from avocado.core.task.runtime import RuntimeTask


class Runtime(TestCase):

    def setUp(self):
        runnable = Runnable('noop', 'noop')
        task = Task('1-noop', runnable)
        self.runtime_task = RuntimeTask(task)

    def test_empty(self):
        self.assertIsNone(self.runtime_task.status)
        self.assertIsNone(self.runtime_task.execution_timeout)
        self.assertIsNone(self.runtime_task.spawner_handle)
        self.assertIsNone(self.runtime_task.spawning_result)

    def test(self):
        self.runtime_task.status = 'LOST CONTACT'
        self.assertEqual(self.runtime_task.task.runnable.kind, 'noop')
        self.assertEqual(self.runtime_task.task.runnable.uri, 'noop')
        self.assertEqual(self.runtime_task.status, 'LOST CONTACT')



from unittest import TestCase

from avocado.core.status import utils


class JSON(TestCase):

    def test_loads_bytes(self):
        self.assertEqual(utils.json_loads(b'{}'), {})

    def test_loads_str(self):
        self.assertEqual(utils.json_loads('{}'), {})

    def test_loads_invalid(self):
        with self.assertRaises(utils.StatusMsgInvalidJSONError):
            utils.json_loads('+-+-InvalidJSON-AFAICT-+-+')

    def test_loads_base64(self):
        data = '{"__base64_encoded__": "dGhpcyBpcyBob3cgd2UgZW5jb2RlIGJ5dGVz"}'
        self.assertEqual(utils.json_loads(data),
                         b'this is how we encode bytes')



import os
import tempfile
import unittest.mock
from urllib.error import URLError

from avocado.core import data_dir
from avocado.core.settings import Settings
from avocado.plugins import vmimage as vmimage_plugin
from avocado.utils import vmimage as vmimage_util

from .. import skipOnLevelsInferiorThan, temp_dir_prefix
from ..functional.test_plugin_vmimage import (create_metadata_file,
                                              missing_binary)

#: extracted from https://dl.fedoraproject.org/pub/fedora/linux/releases/
FEDORA_PAGE = """<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 3.2 Final//EN">
<html>
 <head>
  <title>Index of /pub/fedora/linux/releases</title>
 </head>
 <body>
<h1>Index of /pub/fedora/linux/releases</h1>
<pre><img src="/icons/blank.gif" alt="Icon "> <a href="?C=N;O=D">Name</a>
<a href="?C=M;O=A">Last modified</a>      <a href="?C=S;O=A">Size</a>
<a href="?C=D;O=A">Description</a><hr><img src="/icons/back.gif" alt="[PARENTDIR]">
<a href="/pub/fedora/linux/">Parent Directory</a>                             -
<img src="/icons/folder.gif" alt="[DIR]"> <a href="30/">30/</a>                     2019-04-26 20:58    -
<hr></pre>
</body></html>"""

JEOS_PAGE = """<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 3.2 Final//EN">
<html>
 <head>
  <title>Index of /data/assets/jeos</title>
 </head>
 <body>
<h1>Index of /data/assets/jeos</h1>
<table><tr><th><img src="/icons/blank.gif" alt="[ICO]"></th><th>
<a href="?C=N;O=D">Name</a></th><th><a href="?C=M;O=A">Last modified</a></th><th>
<a href="?C=S;O=A">Size</a></th><th><a href="?C=D;O=A">Description</a></th></tr><tr><th colspan="5"><hr></th></tr>
<tr><td valign="top"><img src="/icons/back.gif" alt="[DIR]"></td><td>
<a href="/data/assets/">Parent Directory</a></td><td>&nbsp;</td><td align="right">  - </td><td>&nbsp;</td></tr>
<tr><td valign="top"><img src="/icons/folder.gif" alt="[DIR]"></td><td><a href="27/">27/</a>
</td><td align="right">11-Dec-2017 17:43  </td><td align="right">  - </td><td>&nbsp;</td></tr>
<tr><th colspan="5"><hr></th></tr>
</table>
</body></html>"""

#: extracted from https://download.cirros-cloud.net/
CIRROS_PAGE = """<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 3.2 Final//EN">
<html>
 <head>
  <title>Index of /</title>
 </head>
 <body>
<h1>Index of /</h1>
<pre>      <a href="?C=N;O=D">Name</a>
<a href="?C=M;O=A">Last modified</a>
 <a href="?C=S;O=A">Size</a>  <a href="?C=D;O=A">Description</a><hr>
 <a href="0.3.0/">0.3.0/</a>                                             2017-11-20 07:20    -

      <a href="0.4.0/">0.4.0/</a>                                             2017-11-19 20:01    -

<hr></pre>
</body></html>"""


class VMImagePlugin(unittest.TestCase):

    def _get_temporary_dirs_mapping_and_config(self):
        """
        Creates a temporary bogus base data dir
        And returns a dictionary containing the temporary data dir paths and
        the path to a configuration file contain those same settings
        """
        prefix = temp_dir_prefix(__name__, self, 'setUp')
        base_dir = tempfile.TemporaryDirectory(prefix=prefix)
        data_directory = os.path.join(base_dir.name, 'data')
        os.mkdir(data_directory)
        os.mkdir(os.path.join(data_directory, 'cache'))
        mapping = {'base_dir': base_dir.name,
                   'data_dir': data_directory}
        temp_settings = ('[datadir.paths]\n'
                         'base_dir = %(base_dir)s\n'
                         'data_dir = %(data_dir)s\n') % mapping
        config_file = tempfile.NamedTemporaryFile('w', delete=False)
        config_file.write(temp_settings)
        config_file.close()
        return base_dir, mapping, config_file.name

    @unittest.mock.patch('avocado.utils.vmimage.urlopen')
    def _create_test_files(self, urlopen_mock):
        with unittest.mock.patch('avocado.core.data_dir.settings', self.stg):
            expected_images = [{'name': 'Fedora', 'file': 'Fedora-Cloud-Base-{version}-{build}.{arch}.qcow2',
                                'url': FEDORA_PAGE},
                               {'name': 'JeOS', 'file': 'jeos-{version}-{arch}.qcow2.xz', 'url': JEOS_PAGE},
                               {'name': 'CirrOS', 'file': 'cirros-{version}-{arch}-disk.img', 'url': CIRROS_PAGE}
                               ]
            cache_dir = data_dir.get_cache_dirs()[0]
            providers = [provider() for provider in vmimage_util.list_providers()]

            for provider in providers:
                for image in expected_images:
                    if image['name'] == provider.name:
                        urlread_mocked = unittest.mock.Mock(return_value=image["url"])
                        urlopen_mock.return_value = unittest.mock.Mock(read=urlread_mocked)
                        image['type'] = "vmimage"
                        image['version'] = provider.version
                        image['arch'] = provider.arch
                        image['build'] = "1234"
                        image['file'] = os.path.join(cache_dir, image['file'].format(
                            version=image['version'],
                            build=image['build'],
                            arch=image['arch']))
                        open(image["file"], "w").close()
                        create_metadata_file(image['file'], image)
            return sorted(expected_images, key=lambda i: i['name'])

    def setUp(self):
        (self.base_dir, self.mapping,
         self.config_file_path) = self._get_temporary_dirs_mapping_and_config()
        self.stg = Settings()
        with unittest.mock.patch('avocado.core.stgs', self.stg):
            import avocado.core
            avocado.core.register_core_options()
        self.stg.process_config_path(self.config_file_path)
        self.stg.merge_with_configs()
        self.expected_images = self._create_test_files()

    def test_list_downloaded_images(self):
        with unittest.mock.patch('avocado.core.data_dir.settings', self.stg):
            with unittest.mock.patch('avocado.utils.vmimage.ImageProviderBase.get_version'):
                images = sorted(vmimage_plugin.list_downloaded_images(), key=lambda i: i['name'])
                for index, image in enumerate(images):
                    for key in image:
                        self.assertEqual(self.expected_images[index][key], image[key],
                                         "Found image is different from the expected one")

    @skipOnLevelsInferiorThan(2)
    @unittest.skipIf(missing_binary('qemu-img'),
                     "QEMU disk image utility is required by the vmimage utility ")
    def test_download_image(self):
        """
        :avocado: tags=parallel:1
        """
        with unittest.mock.patch('avocado.core.data_dir.settings', self.stg):
            try:
                expected_image_info = vmimage_util.get_best_provider(name="CirrOS")
                image_info = vmimage_plugin.download_image(distro="CirrOS")
            except URLError as details:
                raise unittest.SkipTest(details)
            self.assertEqual(expected_image_info.name, image_info['name'],
                             "Downloaded image is different from the expected one")
            self.assertEqual(expected_image_info.version, image_info['version'],
                             "Downloaded image is different from the expected one")
            self.assertEqual(expected_image_info.arch, image_info['arch'],
                             "Downloaded image is different from the expected one")
            self.assertTrue(os.path.isfile(image_info["file"]),
                            "The image wasn't downloaded")

    def tearDown(self):
        self.base_dir.cleanup()


if __name__ == '__main__':
    unittest.main()
